#!/usr/bin/perl -w
use strict;

# script for re-evaluating the dis-similarity of motifs reported by MotifRanking en ook FuzzyClustering
# MS and MR have been run before, FC is run if results is not available yet 
# MC is run again against the SAME file

# all relevant files are written to /users/sista/mclaeys/INCLUSive_tests/MS-250_50_new/output2
# all relevant performance indicators (pi) are written to folder /users/sista/mclaeys/INCLUSive_tests/MS-250_50_new to a file as named in $ARGV[1] (egPerformanceMotifSuite.txt)


## Based on MotifSuite_benchmark_rev5pi.pl (22 june 2011)
## Date of last changes : 24 july 2011
 

# perl command for this script :
#################################################### 
# perl MotifSuite_benchmark.pl $ARGV[0]
####################################################
# $ARGV[0] = settingsfile that was used for the motif-detection algorithm eg run_250_50_new_bench.txt
# $ARGV[1] = name of the file where you want to output results to e.g MC_MR_results.txt
# $ARGV[2] = name of the file where you want to report progress e.g ReportProgress_bench.txt

####################################################
# subroutines
#################################################### 
# &time &readsettings &annotatedhash &countmatrices &gettfbsinfo &validateTFBS &copymotif &indicators &motifcomparison &motiflocator
####################################################


#design :
#voor elke dataset, voor een volledig set van parameters 
#zoek de MS results op voor de gegeven parameter settings voor prior (uniform of fixed, -S is altijd 1)
#zoek MR op en run MotifComparison met de bekomen file als input en output 
#doe alles 10 keer opnieuw

#performantie indicatoren :
#MR : rank, F, PPV, sens, simils, RR, LL, CS
#MC :  KL, BLiC
#rapporteer alles naar txt file, maak avg in excell.

my $driverpath = '/users/sista/mclaeys/INCLUSive_for_users/';
## inclusive version (for MotifComparison)
$driverpath .= 'INCLUSive-3.1.5/';
my $command;
my $fastapath = '/users/sista/vstorms/Datasets/Marleen_MotifSuite/250_50_new/';

# path
my $outputpath;
if ( $driverpath =~ m/sista/) { $outputpath = '/users/sista/mclaeys/INCLUSive_tests/MS-250_50_new/';}
else  { $outputpath = '/home/marleen/MS-250_50_new/';}


# variables to be loaded from the MS-settingsfile
my @data; #= ('LexA');
#my @space; # = ('coregulation/');
my @bgOrder; # = (0,1,2,3, Ecoli_2);
my $bmatrix; # = 1
my $btrack; # = 1;
my @nbrInstPrior; # = ('0.25_0.25','0.9_0.25', 'u', 'e0_1', 'e0_0_1', 'e0_0.45_0.3_0.25','f0_1');
my @search; # = (0,1); 
my $runs; # = 100;
my $repeats; #10
# removed/changed from settingsfile : 
my $width = 0; #to be read from the specific inputfile wM.txt
my $nbrMotifs = 1; 
my $overlap ='-'; # not applicable 
my $strand = 0;
my $maxInst = 0; #to be read from the specific inputfile
# for MR :
my @parMRrank; # = (5); # read from settingsfile
#my @parMRshift = (); # not determing
#my @parMRovl = (); #determing : compute the value based on $width
my @parMRmode = ();# not used in settingsfile
my @parMRtresh = (); # not used in settingsfile
# for FC :
my @parFCmode; # = ('basic', 'sym', 'asym'); #read from settingsfile

# read the ARGV 
my $settingsfile = $ARGV[0];
if (!(-e $settingsfile)) { print 'ERROR '.$settingsfile." : file not found - end\n"; exit(-1);} 
my $pifile = $outputpath.$ARGV[1]; # $outputpath.'PerformanceMotifSuite.txt';
if (!($pifile =~ m/\.txt/))
{ print 'ERROR '.$pifile.' : $ARGV[1] does not have correct pifilename (.txt) - '."end\n"; exit(-1);} 
my $reportfile = $outputpath.$ARGV[2]; 
if (!($reportfile =~ m/\.txt/))
{ print 'ERROR '.$reportfile.' : $ARGV[2] does not have correct pifilename (.txt) - '."end\n"; exit(-1);} 

# read the settings to reconstruct format of the MS/MR/FC-inputfiles
&readsettings($settingsfile);

# for MR :
my $MRrank= $parMRrank[0]; #there is only one value applicable for all
my $shift = 0; # to be set as $width !!
my $ovlap = 0; # to be set as half of $width !!
my $tresh = 0.4; # the general default 
# not used in settingsfile, but in case it would be used :::::
#if (scalar(@parMRtresh) == 1){ $tresh = $parMRtresh[0];}
my $treshpr = $tresh; $treshpr =~ s/\.//;
my $mode = 0; 
#if (scalar(@parMRmode) == 1){ $mode = $parMRmode[0];}

# for FC :

# for MotifComparison
my $ps = '0'; # no pseudo used for regDB

# for MotifLocator



# local variables
#my $path;
my $newpath; # use the output2-path
my $bOverlap;
my $wMfile;
my $inputfile;
my $matrixfile;
my $textfile;
my $MSkolom;
my $MRkolom;
my $FCkolomb;my $FCkoloms;my $FCkoloma;
my $prior; 
my $method; 

my $totalInst;
my $annotatie;
my $bgfile;
my $MSconv;
my @split;
my $rank; my $sim; my$F, my$PPV; my$sens; my$CS; my$LL; my $nbrInst;
my $tfbsgo; my $seq; my$start; my$true; my$tp; my$fp; my$nbrmotifs;
my $weight; my$aWpos; my$aWneg; my$aW; my$minW; my$maxW;
my $occ; my$aOccpos; my$aOccneg; my$aOcc; my$minOcc; my$maxOcc;
my $text1; my $text;
my $copymtrx; my $copytxt;  my $perf; my @mloutput;
my $o; my $file; my $line;
my @values; my$convergence; my$runsMR1;my$runsMR2;my@runsFC1;my@runsFC2;
my @MR12; for( my $q=0; $q < 36; $q++) { push(@MR12, 0);} #=(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0); #sim,F,PPV,sens,CS,LL, ML012:F,PPV,sens,nbrInst  # = (6+12) * 2
my @FCa12; for( my $q=0; $q < 52; $q++) { push(@FCa12, 0);}# = (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);#F,PPv,sens,CS,aW,aW+,aW-,minW,maxW,aOcc,aOcc+,aOcc-,minOcc,maxOcc,ML012:F,PPV,sens,nbrInst # = (14+12) * 2

my $now = &time;
# file to write pi results
open(FO,">$pifile");
print FO "Time : $now\n";
print FO "MS-MR-MC\tTF\trun\tbg\tprior\tsearch\tMSconv\tMRrk\tsimils\tF\tPPV\tsens\tCS\tLL\tDBid\tKL\tpBLiC\n";
close(FO);

# file to write runnings
open(FREP,">$reportfile");
print FREP "Time : $now\n";




# repeat for each dataset
for(my $d = 0; $d < scalar(@data); $d++)
{
  $width = 0;$maxInst = 0;$totalInst = 0;
  # get the motifwidth and maxInst (the wM.txt file should be available)
  $wMfile = $outputpath.'RegulonDB/'.$data[$d].'_wM.txt';
  if (!(-e $wMfile))
  { next;} #no printerror needed anymore - see rev1 for that
  &readsettings($wMfile);
  if ( $width == 0 || $maxInst == 0)
  { next;} #no printerror needed anymore - see rev1 for that
  $ovlap = int($width/2);
  $shift = $width;

  # repeat for each backgroundfile
  for(my $b = 0; $b < scalar(@bgOrder); $b++)
  {
    # compose bgfile name (MotifComparison BLiC and MotifLocator)
    if ( $bgOrder[$b] =~ m/Ecoli/){ $bgfile = $outputpath.'RegulonDB/'.$bgOrder[$b].'.bg'; }
    else { $bgfile = $outputpath.'RegulonDB/'.$data[$d].'_'.$bgOrder[$b].'.bg'; }

    #repeat for each nbrInstprior
    for(my $p = 0; $p < scalar(@nbrInstPrior); $p++)
    {
      $prior = $nbrInstPrior[$p];
      #repeat for each search method
      for(my $m = 0; $m < scalar(@search); $m++)
      {
        $method = $search[$m];
        if ($prior =~ m/^f0/ && $method == '0') 
        {#print "WARNING fixed prior not run for search 0 - SKIP\n"; 
         next;} 

        $newpath = $outputpath.'output2/'.$data[$d].'/'.'bg_'.$bgOrder[$b].'/prior_'.$prior.'/method_'.$method;
        if (!(-e $newpath))
        { print FREP "Rerun all on $newpath\n"; next;}

        ##reset variables to compute average over 10 runs
        $convergence = 0;
        $runsMR1 = 0;$runsMR2 = 0; 
        for (my$fc=2;$fc<3;$fc++){$runsFC1[$fc] =0;$runsFC2[$fc] =0;}#for sym #not basic/asym
        for (my$c=0;$c<scalar(@MR12);$c++) { $MR12[$c] = 0;}
        for (my$c=0;$c<scalar(@FCa12);$c++) { $FCa12[$c]= 0;}


        #now find the requested MS mtrxfile and MR txt/mtrx files and FC txt/mtrx files
        for(my $r=1; $r <= $repeats; $r++)
        {
          # put all txt and mtrx file names in an array for MS,MR,FC(3)
          my @txtfiles = ('MStxt','MRtxt','FCbtxt','FCstxt','FCatxt');
          my @mtrxfiles = ('MSmtrx','MRmtrx','FCbmtrx','FCsmtrx','FCamtrx');

          $MSkolom = 'w'.$width.'_n'.$nbrMotifs.'_s'.$strand.'_x'.$overlap.'_M'.$maxInst.'_r'.$r;
          $txtfiles[0] = $newpath.'/'.$MSkolom.'.txt';  $mtrxfiles[0] = $newpath.'/'.$MSkolom.'.mtrx'; # output2 !.  
          if (! (-e $mtrxfiles[0]))
          { print FREP 'Run MS needed (not found): '. $mtrxfiles[0]." - SKIP\n"; next; }
        
          $MRkolom = '_MR_s'.$shift.'_x'.$ovlap.'_r'.$MRrank.'_m'.$mode.'_t'.$treshpr;
          $txtfiles[1] = $newpath.'/'.$MSkolom.$MRkolom.'.txt';  $mtrxfiles[1] = $newpath.'/'.$MSkolom.$MRkolom.'.mtrx';
          if (! (-e $mtrxfiles[0]))
          { print FREP 'Run MR needed (not found): '. $mtrxfiles[0]." - SKIP\n"; next; }

          # $FCkolomb = '_FC_tbasic_i05_c025_b1_p01_m05';
          # $txtfiles[2] = $newpath.'/'.$MSkolom.$FCkolomb.'.txt';  $mtrxfiles[2] = $newpath.'/'.$MSkolom.$FCkolomb.'.mtrx';
          # $FCkoloms = '_FC_tsym_i05_c025_b1_p01';
          # $txtfiles[3] = $newpath.'/'.$MSkolom.$FCkoloms.'.txt';  $mtrxfiles[3] = $newpath.'/'.$MSkolom.$FCkoloms.'.mtrx';
          $FCkoloma = '_FC_tasym_i05_c025_j01';
          $txtfiles[4] = $newpath.'/'.$MSkolom.$FCkoloma.'.txt';  $mtrxfiles[4] = $newpath.'/'.$MSkolom.$FCkoloma.'.mtrx';
          if (! (-e $mtrxfiles[0]))
          { print FREP 'Run FC needed (not found): '. $mtrxfiles[4]." - SKIP\n"; next; }

          print FREP 'OK for '. $newpath." - GOOOOO\n";
          ########## MS : pi=convergence by number of putative motifs
          $MSconv = &countmatrices($mtrxfiles[0]);
          $text1 = $data[$d]."\t".$r."\t".$bgOrder[$b]."\t".$prior."\t".$method."\t".$MSconv;
          if ($MSconv <= 0)
          {
            print $text1;
            open(FOOF,">>$pifile");
            print FOOF $text1."\tend\n";
            close(FOOF);
          }
          else
          {
            #add to resume
            $convergence += $MSconv;
            $runsMR1++;             

            #for next-lines reporting
            $text1 = $data[$d]."\t".$r."\t".$bgOrder[$b]."\t".$prior."\t".$method."\t".$MSconv;

            ############# MR :
            # get required TFBS-info from the MS-txtfile - with the motif/TFBSs in ranked order !
            my $txt2file = &gettfbsinfo($txtfiles[1]);
            # for each motif : read nbr simils, if > 10 ==> run MC 
            # read through MR-txt file with #similars
            $textfile = $txtfiles[1]; $rank = 0;
            open(FI, "<$textfile");
            while(<FI>)
            { 
              $line = $_;
              if ($line =~ m/^\=\=\>\sTotal/) # similars information eg ==> Total: 22
              {
                $rank++;
                @split = split(/\s+|\t+/,$line);
                $sim = $split[2];
                if ($rank == 1 || $sim >= 10) ## always print the first and others only if more than 10 simils
                {
                  if ($rank == 2){$runsMR2++;}
                  # copy this motif from into a separate file, to be deleted afterwards
                  $copymtrx = &copymotif($mtrxfiles[1], $rank);
                  $text = $text1."\tMR_".$rank."\t".$sim; 

                  #run MotifComparison for this MR-motif against ITSELF and write results to pi-file
                  #########################################################################################
                  &motifcomparison($copymtrx, $pifile, $width, $bgfile, $text, $mtrxfiles[1]);

                  ##delete copymatrix
                  $command = 'rm -f '.$copymtrx; system($command);

                } #end for this ranked motif
              } # end for reading similar-lines
            } ## reading through MR-file

            close(FI);# close MR txt file 
            #cleanup txt2file
            $command = 'rm -f '.$txt2file; system($command);

            ############# FC :
            # for each motif : compute F,PPV,sens,aW+,aW-,aW,minW,maxW,aOcc+,aOcc-,aOcc,minOcc,maxOcc, read CS, compute KL,BLiC tov regDBdb
            # read through FC mtrx files (asym)   not basic/sym/
            for(my$fc=2;$fc<3;$fc++)
            { 
              $matrixfile = $mtrxfiles[$fc+2]; 
              $nbrmotifs = &countmatrices($matrixfile);
              if ($nbrmotifs > 0) {$runsFC1[$fc]++;}
              if ($nbrmotifs > 1){$runsFC2[$fc]++;}

              $textfile = $txtfiles[$fc+2];
              #repeat for each motif
              for($rank = 1; $rank <=$nbrmotifs;$rank++)
              { 
                #store info in copytxt for verfication (temporary)
                $text = $text1."\tFC_".$fc.'_'.$rank;

                #copy this motif PWM into a separate file, to be deleted afterwards
                $copymtrx = &copymotif($matrixfile, $rank);

                #run MotifComparison for this FC-motif against ITSELF and write results to pi-file
                #########################################################################################
                &motifcomparison($copymtrx, $pifile, $width, $bgfile, $text, $matrixfile);

                ##delete copymatrix and copytext
                $command = 'rm -f '.$copymtrx; system($command);

              } #end for this ranked motif
            } ## end for this fc-type file
          } # if conv > 0
          # report progress 
          ##if ($r == $repeats){ print $data[$d]."-".$bgOrder[$b]."-".$prior."-".$method."-done\n";}   
        } #end of this repeat
      } #end of this search
    } #end of this prior
  } #end of this bgmodel
  print 'INFO @data[$d] = '.$data[$d]." - FINISHED\n";  
} #end of this regulator
close(FREP);
print 'END script finishes succesfully'."\n";         
           
######################################## SUBROUTINES
######################################################################################
# local time in format 0000-00-00 00:00:00
######################################################################################
sub time
{ 
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,
      $yday,$isdst)=localtime(time);
  my $format = sprintf ("%4d-%02d-%02d %02d:%02d:%02d",$year+1900, $mon+1,$mday,$hour,$min,$sec); 
  return($format);
}
######################################################################################
#loading variables from file into main
######################################################################################
sub readsettings 
{ 
  my ($file) = @_;
  open(FI, $file);
  while(<FI>) 
  {
    my $line = $_; 
    chomp $line;
    if ($line =~ m/^\$/ || $line =~ m/^\@/){ eval($line);}
  }
  close(FI);
}


######################################################################################
#count the number of matrices in a matrixfile
######################################################################################
sub countmatrices # ($D1) = &countmatrices($matrixfile);
{
  my ($fName) = @_; 
  open(FI, "<$fName");
  my $count = 0; my $motif = ''; my @split;
  while(<FI>)
  { 
    my $line = $_;
    if ($line =~ m/^#ID/) #line of motif #ID = box_1_1_TCGATAACnTGTT_[PWM=CS-based]
    { # for fc, both CS as W may be reported : count this only once ! %2009/12/02
      @split = split("PWM",$line);
      if (@split >1) #this means it is fc-output
      {
        if ( $split[0] eq $motif) {next;} #do not count the second W-motif 
        $motif = $split[0];
      }
      $count++;
    }
  } 
  close(FI); 
  return($count);
}



######################################################################################
#copy rank-motif from a file into a separate file, applicable for TFBS for MR and PWM for MR/FC
######################################################################################
sub copymotif # $copymotif = &copymotif($mtrxfiles[1], $rank);
{
  my($file, $rk) = @_;

  #get the requested (rk) motif from file :make a copy of all info untill next motif to file.sub 
  ## for FC-MATRIX-output, only consider the CS-motif in case also W-motif is printed 
  my $copy = $file.'sub'.$rk;
  open(FOM, ">$copy");
  print FOM "#INCLUSive Motif Model\n#\n";
  open(FIM, "<$file");
  my $rkk = 0; my $go = 0; my @spl; my $print = 0;
  while(<FIM>)
  { 
    my $lin = $_;
    if ($lin =~ m/^#ID/ || $lin =~ m/^#id/) #line of motif 
#format MR-tfbs : #id: box_31_1_wkwACTGTATATnnAynCAG	cons...
#format FC-mtrx : #ID = box_1_CS[...]_W[...]_E=0.329804_nbrInst=8_[PWM=CS-based] or _[PWM=width-based]
#format MR-mtrx : #ID = box_31_1_wkwACTGTATATnnAynCAG
    { # for fc, both CS as W may be reported : count and copy only CS-based (printed first)
      @spl = split("PWM=",$lin);
      if (@spl > 1) #this means it is double FC-output
      { if ( $spl[1] =~ m/CS/ ) {$rkk++;}
        else { $go = 0; if ($rkk == $rk){last;}} #CS-motif is already done, skip W representation.
      } # this is MR or single FC output
      else {$rkk++;} 
      # get the right motif
      if ($rkk < $rk) {next;}
      elsif ($rkk == $rk) { $go = 1;}
      else {last;}
    }
    if ($go) { print FOM $lin; $print = 1;} #copy each line
  } 
  close(FIM); 
  close(FOM);
  if (!$print)
  { print 'WARNING '.$copy." : copyfile does not contain a motif - DEBUG sub copymatrix - exit!\n"; exit(-1);}
  return($copy);
}



######################################################################################
#run motifcomparison with KL or BLiC for one motif, print +similarity results to file
######################################################################################
sub motifcomparison ## &motifcomparison($copymtrx, $pifile, $width, $bgfile, $text);
{
  my($matrices2, $druk, $w, $b, $txt, $regDB) = @_; # compare to the MR file ITSELF
  my$s = $w; #shift
  my$x = int($w/2); #overlap
  #my $regDB = $outputpath.'RegulonDB_freq_6-4.mtrx'; #path to independently created full regDB  

  ## run the MC commando KL
  my $KLfile = $matrices2.'KLself';
#if (!(-e $KLfile))
#{
  my $command = $driverpath.'applications/MotifComparison -m '.$matrices2.' -o '.$KLfile.' -d '.$regDB.' -t 0.4 -s '.$s.' -x '.$x.' -l 0';
  system($command);
#}
  ## run the MC commando BLiC
  my $BLiCfile = $matrices2.'BLiCself';
#if (!(-e $BLiCfile))
#{
  $command = $driverpath.'applications/MotifComparison -m '.$matrices2.' -o '.$BLiCfile.' -d '.$regDB.' -t 0.001 -s '.$s.' -x '.$x.' -l 1 -n '.$w.' -b '.$b; #no entry for -p (Dirichlet default)
  system($command);
#}

  ## write similarities to $pifile
#KLformat : #query:query_consensus	match:match_consensus	score;(shiftm-d,shiftd-m)
#box_1_1_ATTCCTACnnnTGTArGA:ATTCCTACnnnTGTArGA	box_ann1_ps0.001_TTCCTACAsyTGTAAGAAw:TTCCTACAsyTGTAAGAAw	0.270935;(-1,1)
#BLiCformat : query:query_consensus	match:match_consensus	score;shift;overlap;strand	p-value;common;(n:avg,stdev)
#box_1_1_ATTCCTACnnnTGTArGA:ATTCCTACnnnTGTArGA	box_ann1_ps0.001_TTCCTACAsyTGTAAGAAw:TTCCTACAsyTGTAAGAAw	20.5;-1;17;fwd	5.3301e-07;0;(n=18:9.07,1.06)
  open(INPUT1, "<$KLfile"); 
  open(INPUT2, "<$BLiCfile"); 
  open(FOOT,">>$pifile");
  my $line1; my $line2; my @result1; my $id; my $regid; my @result2; my $KL; my $pBLiC; my @spl;
  while ( <INPUT1>) 
  {
    $line1 = $_;
#print "line 1 = $line1";
    $line2 = <INPUT2>; #read the same line in other file
#print "line 2 = $line2";
    #if ($line1 =~ m/^\#/) {next;} # masked lines for distant matrices / also read incorrect matrices to track topKL/topBLiC.
    if ($line1 =~ m/box/) # this is a similairty line (correct or incorrect)
    {
      #if ( !($line1 =~ m/^\#/ && $line2 =~ m/^\#/)) # one or both show similarity 
      if (1)
      { 
        @result1 = split(/\t/, $line1); #KL-line
        @result2 = split(/\t/, $line2); #pBLiC-line
        @spl = split(/\:/, $result1[0]); $id = $spl[0];
        @spl = split(/\:/, $result1[1]); $regid = $spl[0]; @spl = split(/\:/, $result2[1]);
        if ($regid ne  $spl[0]) {print 'ERROR - sub MCevalutation : error in file-reading1 - debug $regidKL = '.$regid.' - regidBLiC = '.$spl[0].' - exit'."\n"; exit(-1);}
        # get KL score
        @spl = split(/\;/,$result1[2]); $KL = $spl[0];
        # get pvalue BLiC score
        @spl = split(/\;/,$result2[3]); $pBLiC = $spl[0];
        # print to perf file
        if ($pBLiC <= 0.001 && !($regid =~ m/$id/))
        { print FOOT $txt."\t".'id1='.$id."\t".'id2='.$regid."\t".$KL."\t".$pBLiC."\n";    }
      }
      # go to next line in both files 
    }
  }
  close(INPUT1);
  close(INPUT2);
  close(FOOT);
  return;
}
######################################################################################
# extract from MStxtfile those motifs that belong to MRtxtfile, in the correct ranked order
######################################################################################
sub gettfbsinfo #$txt2file = &gettfbsinfo($txtfile);
{
  my ($MRfile) = @_; 
  my @splitt= split("_MR", $MRfile); my $MSfile = $splitt[0].'.txt';
  if (!(-e $MSfile))
  { print 'WARNING '.$MSfile." : file not found - DEBUG sub gettfbsinfo - exit!\n"; exit(-1);}

  my $txt2 = $MRfile.'MRsub';

  my @split; my $motifid; my $id; my $go = 0;

  open(FOUT, ">$txt2");
  print FOUT '#INCLUSive GFF File'."\n#\n";

  # read through MRfile
  open(FI, "<$MRfile");
  while(<FI>)
  { 
    my $line = $_;
    if ($line =~ m/^\t/ || $line =~ m/^\#/) {next;}
    if ($line =~ m/^\d+\)/) #line of ranked motif 1)box_31_1_wkwACTGTATATnnAynCAG	126.981
    { @split = split(/\)|\s+|\t+/,$line);
      $motifid = $split[1]; 
      # find this motif in MStextfile and copy all tfbs lines
      $go = 0;
      open(FI2, "<$MSfile");
      while(<FI2>)
      { 
        my $linee = $_;
        if ($linee =~ m/^\#id/) 
        { #(#id: box_31_1_wkwACTGTATATnnAynCAG	cons...)
          @split = split(/\t+|\s+/,$linee);
          $id = $split[1];
          if ($id eq $motifid) {$go = 1;} elsif ($go == 1){ last;}
        }
        if ($go) #print instances
        { print FOUT $linee;}   
      }
      close(FI2);
      if (!$go)
      { print 'WARNING '.$motifid.' : not found in '.$MSfile." - DEBUG sub gettfbsinfo - exit!\n"; exit(-1);}
    }
  } 
  close(FI);
  close(FOUT);
 
  return($txt2); 
}


