#!/usr/bin/perl -w
use strict;

# script for MotifSuite benchmark 
# MS and MR have been run before, FC is run if results is not available yet 
# MC is run again against the complete database in FREQ and in COUNT format (instead of only the known motif with 0.001 pseudocount)
# ML is run on all retained motifs from MR and FC (if not yet available)

# all relevant files are copied/written to /users/sista/mclaeys/INCLUSive_tests/MS-250_50_new/output1
# all relevant performance indicators (pi) are written to /users/sista/mclaeys/INCLUSive_tests/MS-250_50_new/PerformanceMotifSuite.txt 

## Date of last changes : 10 nov 2010 (rev1: copy output->output1)

# perl command for this script :
#################################################### 
# perl MotifSuite_benchmark.pl $ARGV[0]
####################################################
# $ARGV[0] = settingsfile that was used for the motif-detection algorithm eg run_250_50_new_bench.txt

## mark that this script is designed for one motif
## and for fixed motif length

#design :
#voor elke dataset, voor een volledig set van parameters 
#zoek de MS results op
#zoek MR op en bereken PI voor alle met simils groter dan 10
#other script : run FuzzyClustering met c 0.25 en basic m0.5p01 en sym p01 en asym(full noise) j 0.1, bereken PI voor alle
#run MotifComparison met retained predictie tov full database 
#run MotifLocator met retained predictie (voor database motif ook met T=0.9)

#doe alles 10 keer opnieuw

#performantie indicatoren :
#MS : nbr motifs -> convergence
#MR : rank, F, PPV, sens, simils, RR, LL, CS
#FC : rank, F, PPV, sens, CS, aOcc/minOcc/maxOcc, aWeight/minW/maxW
#MC : DB-ID, KL, BLiC
#ML : F, PPV, sens
#rapporteer alles naar txt file, maak avg in excell.

my $driverpath = '/users/sista/mclaeys/INCLUSive_for_users/';
## inclusive version (for MotifComparison)
$driverpath .= 'INCLUSive-3.1.5/';
my $command;

# path
my $outputpath;
if ( $driverpath =~ m/sista/) { $outputpath = '/users/sista/mclaeys/INCLUSive_tests/MS-250_50_new/';}
else  { $outputpath = '/home/marleen/MS-250_50_new/';}

# file with pi results
my $pifile = $outputpath.'PerformanceMotifSuite.txt';


# variables to be loaded from the MS-settingsfile
my @data; #= ('LexA');
#my @space; # = ('coregulation/');
my @bgOrder; # = (0,1,2,3, Ecoli_2);
my $bmatrix; # = 1
my $btrack; # = 1;
my @nbrInstPrior; # = ('0.25_0.25','0.9_0.25', 'u', 'e0_1', 'e0_0_1', 'e0_0.45_0.3_0.25','f0_1');
my @search; # = (0,1); 
my $runs; # = 100;
my $repeats; #10
# removed/changed from settingsfile : 
my $width = 0; #to be read from the specific inputfile wM.txt
my $nbrMotifs = 1; 
my $overlap ='-'; # not applicable 
my $strand = 0;
my $maxInst = 0; #to be read from the specific inputfile
# for MR :
my @parMRrank; # = (5); # read from settingsfile
#my @parMRshift = (); # not determing
#my @parMRovl = (); #determing : compute the value based on $width
my @parMRmode = ();# not used in settingsfile
my @parMRtresh = (); # not used in settingsfile
# for FC :
my @parFCmode; # = ('basic', 'sym', 'asym'); #read from settingsfile

# read the ARGV 
my $settingsfile = $ARGV[0];
if (!(-e $settingsfile)) { print 'ERROR '.$settingsfile." : file not found - end\n"; exit(-1);} 
# read the settings to reconstruct format of the MS/MR/FC-inputfiles
&readsettings($settingsfile);

# for MR :
my $MRrank= $parMRrank[0]; #there is only one value applicable for all
my $shift = 0; # to be set as $width !!
my $ovlap = 0; # to be set as half of $width !!
my $tresh = 0.4; # the general default 
# not used in settingsfile, but in case it would be used :::::
#if (scalar(@parMRtresh) == 1){ $tresh = $parMRtresh[0];}
my $treshpr = $tresh; $treshpr =~ s/\.//;
my $mode = 0; 
#if (scalar(@parMRmode) == 1){ $mode = $parMRmode[0];}

# for FC :


# for MotifComparison
my $ps = '0'; # $ps -> to read info from wM.txt file 

# for MotifLocator




#start with copying the MS and MR files (from folder output->output1)
# and also write the applied MS parameter settings to file /users/sista/mclaeys/INCLUSive_tests/MS-250_50_new/MSparameters.txt
my $parfile = $outputpath.'MSparameters.txt';
open(FOT,">$parfile");

# local variables
my $path;
my $newpath;
my $bOverlap;
my $wMfile;
my $inputfile;
my $matrixfile;
my $textfile;
my $newfile;
my $MSkolom;
my $MRkolom;
my $prior; 
my $method; 

# repeat for each dataset
for(my $d = 0; $d < scalar(@data); $d++)
{
  # get the motifwidth and maxInst (the wM.txt file should be available)
  $wMfile = $outputpath.'RegulonDB/'.$data[$d].'_wM.txt';
  if (!(-e $wMfile))
  { print FOT 'WARNING '.$wMfile." : wMfile not found - SKIP transcription factor\n"; next;}
  &readsettings($wMfile);
  if ( $width == 0 || $maxInst == 0)
  { print FOT 'WARNING '.$wMfile.' : wrong $width/$maxInst format - SKIP transcription factor'."\n"; next;} 
  $ovlap = int($width/2);
  $shift = $width;

  # repeat for each backgroundfile
  for(my $b = 0; $b < scalar(@bgOrder); $b++)
  {
    #repeat for each nbrInstprior
    for(my $p = 0; $p < scalar(@nbrInstPrior); $p++)
    {
      $prior = $nbrInstPrior[$p];
      #repeat for each search method
      for(my $m = 0; $m < scalar(@search); $m++)
      {
        $method = $search[$m];
        $path = $outputpath.'output/'.$data[$d].'/'.'bg_'.$bgOrder[$b].'/prior_'.$prior.'/method_'.$method; #here still in 'output'
        if (!(-e $path))
        { print FOT 'WARNING '.$data[$d].'/'.'bg_'.$bgOrder[$b].'/prior_'.$prior.'/method_'.$method." : dir not found - SKIP transcription factor\n"; next;}
        $newpath = $path;
        $newpath =~ s/output/output1/; #here in 'output1'
        if (!(-e $newpath))
        {$command = 'mkdir -p '.$newpath; system($command);}

        #now find the requested MS txtfile and MR txt/mtrx files for copying
        for(my $r=1; $r <= $repeats; $r++)
        {
          $MSkolom = 'w'.$width.'_n'.$nbrMotifs.'_s'.$strand.'_x'.$overlap.'_M'.$maxInst.'_r'.$r;
          $MRkolom = '_MR_s'.$shift.'_x'.$ovlap.'_r'.$MRrank.'_m'.$mode.'_t'.$treshpr;
          # MS txt file
          $inputfile = $path.'/'.$MSkolom.'.txt';
          if (!(-e $inputfile))
          { print 'WARNING '.$inputfile." : file not found - SKIP copy\n"; next;}
          &copyfile($inputfile);
          # MS mtrx file
          $inputfile = $path.'/'.$MSkolom.'.mtrx';
          if (!(-e $inputfile))
          { print 'WARNING '.$inputfile." : file not found - SKIP copy\n"; next;}
          &copyfile($inputfile);
          # MR txt file
          $inputfile = $path.'/'.$MSkolom.$MRkolom.'.txt';
          if (!(-e $inputfile))
          { print 'WARNING '.$inputfile." : file not found - SKIP copy\n"; next;}
          &copyfile($inputfile);
          # MR mtrx file
          $inputfile = $path.'/'.$MSkolom.$MRkolom.'.mtrx';
          if (!(-e $inputfile))
          { print 'WARNING '.$inputfile." : file not found - SKIP copy\n"; next;}
          &copyfile($inputfile);
          # report progress 
          if ($r == $repeats){print FOT $data[$d]."\t".$bgOrder[$b]."\t".$prior."\t".$method."\n";}   
        }
      }
    }
  }
  print 'INFO @data[$d] = '.$data[$d]." - FINISHED\n";  
}
print 'END script finishes succesfully'."\n";
close(FOT);         
           
######################################## SUBROUTINES
######################################################################################
#loading variables from file into main
######################################################################################
sub readsettings 
{ 
  my ($file) = @_;
  open(FI, $file);
  while(<FI>) 
  {
    my $line = $_; 
    chomp $line;
    if ($line =~ m/^\$/ || $line =~ m/^\@/){ eval($line);}
  }
  close(FI);
}
######################################################################################
#copy file from output to output1 folder (open, read1 and write2, close)
######################################################################################
sub copyfile
{ 
  my ($file1) = @_;
  my $file2 = $file1;
  $file2 =~ s/output/output1/;
  open(FI, $file1);
  open(FOO,">$file2");
  while(<FI>) 
  {
    my $line = $_; 
    print FOO $line;
  }
  close(FI);
  close(FOO);
}
