#!/usr/bin/perl -w
use strict;

# script for MotifSuite benchmark 
# MS has been run before, 
# FC is run again with version FC-5.2.0 

# all relevant files are copied/written to /users/sista/mclaeys/INCLUSive_tests/MS-250_50_new/output
# all relevant performance indicators (pi) are written to folder /users/sista/mclaeys/INCLUSive_tests/MS-250_50_new to a file as named in $ARGV[1] (eg PerformanceMotifSuite.txt)

## Date of last changes : 10 nov 2010 (rev1: copy output->output1)
## Date of last changes : 11 nov 2010 (rev2: compute pi)
## Date of last changes : 17 jan 2011 (add argv[1] for outputfile name)
## Date of last changes : 2 march 2011 (error in line 862 ($line2 instead of $line1)- see documented in book)
## Date of last changes : 14 june 2011 : copy requested folders to output2 (to cleanup data on server)  -- all other settings not mentioned in the settingsfile are cleaned up (remove output1 manually afterwards). 
## + in case prior settings = fixed : only run with search=1 (ignore search=0).
## + report if folder not found (for check if running still needed of not) + timing of operation (to distinguish and archive)
## + only do FC asym, not the basic and sym mode (this means all these data will not be transferred to output2 also)
## Date of last changes : 22 june 2011 : output1 folder deleted, remove its use in this script
## Date of last changes : 16 nov 2011 : only FC, en bereken andere indicators, output2->again output1

# perl command for this script :
#################################################### 
# perl MotifSuite_benchmark.pl $ARGV[0]
####################################################
# $ARGV[0] = settingsfile that was used for the motif-detection algorithm eg run_250_50_new_bench.txt
# $ARGV[1] = name of the file where you want to output results to e.g PerformanceMotifSuite_part1.txt
# $ARGV[2] = name of the file where you want to report progress e.g ReportProgress_bench.txt

####################################################
# subroutines
#################################################### 
# &time &readsettings &annotatedhash &countmatrices &gettfbsinfo &validateTFBS &copymotif &indicators &motifcomparison &motiflocator
####################################################


#design :
#voor elke dataset, voor een volledig set van parameters 
#zoek de MS results op en run FC

#doe alles 10 keer opnieuw

#performantie indicatoren :
#MS : nbr motifs -> convergence
#FC : rank, F, PPV, sens, CS, aOcc/minOcc/maxOcc, aWeight/minW/maxW, N
#rapporteer alles naar txt file, maak avg in excell.

my $driverpath = '/users/sista/mclaeys/INCLUSive_for_users/';
$driverpath .= 'INCLUSive-3.1.5/';
my $command;
my $fastapath = '/users/sista/vstorms/Datasets/Marleen_MotifSuite/250_50_new/';

# path
my $outputpath;
if ( $driverpath =~ m/sista/) { $outputpath = '/users/sista/mclaeys/INCLUSive_tests/MS-250_50_new/';}
else  { $outputpath = '/home/marleen/MS-250_50_new/';}


# variables to be loaded from the MS-settingsfile
my @data; #= ('LexA');
#my @space; # = ('coregulation/');
my @bgOrder; # = (0,1,2,3, Ecoli_2);
my $bmatrix; # = 1
my $btrack; # = 1;
my @nbrInstPrior; # = ('0.25_0.25','0.9_0.25', 'u', 'e0_1', 'e0_0_1', 'e0_0.45_0.3_0.25','f0_1');
my @search; # = (0,1); 
my $runs; # = 100;
my $repeats; #10
# removed/changed from settingsfile : 
my $width = 0; #to be read from the specific inputfile wM.txt
my $nbrMotifs = 1; 
my $overlap ='-'; # not applicable 
my $strand = 0;
my $maxInst = 0; #to be read from the specific inputfile

# read the ARGV 
my $settingsfile = $ARGV[0];
if (!(-e $settingsfile)) { print 'ERROR '.$settingsfile." : file not found - end\n"; exit(-1);} 
my $pifile = $outputpath.$ARGV[1]; # $outputpath.'PerformanceMotifSuite.txt';
if (!($pifile =~ m/\.txt/))
{ print 'ERROR '.$pifile.' : $ARGV[1] does not have correct pifilename (.txt) - '."end\n"; exit(-1);} 
my $reportfile = $outputpath.$ARGV[2]; 
if (!($reportfile =~ m/\.txt/))
{ print 'ERROR '.$reportfile.' : $ARGV[2] does not have correct pifilename (.txt) - '."end\n"; exit(-1);} 

# read the settings to reconstruct format of the MS/MR/FC-inputfiles
&readsettings($settingsfile);

# local variables
#my $path;
my $newpath; # use the output2-path
my $bOverlap;
my $wMfile;
my $inputfile;
my $matrixfile;
my $textfile;
my $MSkolom;
my $FCkolom;
my $prior; 
my $method; 

my $totalInst;
my $annotatie;
my $bgfile;
my $MSconv;
my @split;
my $rank; my $sim; my$F, my$PPV; my$sens; my$CS; my$LL; my $nbrInst; my $Nw; my $OccW;
my $tfbsgo; my $motifsgo; my $seq; my$start; my$true; my$tp; my$fp; my $nbrmotifs; my $nbrMot;
my $weight; my$aIMpos; my$aIMneg; my$aIM; my$minIM; my$maxIM;my$aMM; my$minMM; my$maxMM;
my $text1; my $text; my $Ni;
my $copymtrx; my $copytxt;  my $perf; 
my $o; my $file; my $line;
my @values; my$convergence; 
my @FC12; for( my $q=0; $q < 30; $q++) { push(@FC12, 0);}
#Occ(i) N(i) N(m) aIM minIM maxIM aMM minMM maxMM aIMpos aIMneg F PPV sens CS  # = (15) * 2

my $now = &time;
# file to write pi results
open(FO,">$pifile");
print FO "Time : $now\n";
print FO "TF\trun\tbg\tprior\tsearch\tMSconv\tFCtrk\tOcc(i)\tN(i)\tN(m)\taIM\tminIM\tmaxIM\taMM\tminMM\tmaxMM\taIMpos\taIMneg\tF\tPPV\tsens\tCS\tDBid\tKL\tpBLiC\n";
close(FO);

# file to write runnings
open(FREP,">$reportfile");
print FREP "Time : $now\n";

my @resume = (0,0,0); # successful repeats for MS, FC1 and FC2


# repeat for each dataset
for(my $d = 0; $d < scalar(@data); $d++)
{
  $width = 0;$maxInst = 0;$totalInst = 0;
  # get the motifwidth and maxInst (the wM.txt file should be available)
  $wMfile = $outputpath.'RegulonDB/'.$data[$d].'_wM.txt';
  if (!(-e $wMfile))
  { next;} #no printerror needed anymore - see rev1 for that
  &readsettings($wMfile);
  if ( $width == 0 || $maxInst == 0)
  { next;} #no printerror needed anymore - see rev1 for that

  ##
  # create a hash of arrays with results from annotatie-file for this TF
  $annotatie =  $outputpath.'RegulonDB/'.$data[$d].'_annotatie.txt';
  if (!(-e $annotatie))
  { print 'WARNING '.$annotatie." : annotatie file not found - SKIP further performance computation !\n"; next;}
  my ($annotP, $annothashref) = &annotatedhash($annotatie);
  if ( $annotP < 1 ) 
  { print 'WARNING '.$annotatie." : no annotated TFBS information available - SKIP further performance !\n"; next;}
  ## do a little check :
  if ($annotP != $totalInst)
  { print 'WARNING $annotP/'.$annotP.' != $totalInst/'.$totalInst." - CHECK THIS !!!!\n"; }
  ####


  # repeat for each backgroundfile
  for(my $b = 0; $b < scalar(@bgOrder); $b++)
  {
    # compose bgfile name (MotifComparison BLiC and MotifLocator)
    if ( $bgOrder[$b] =~ m/Ecoli/){ $bgfile = $outputpath.'RegulonDB/'.$bgOrder[$b].'.bg'; }
    else { $bgfile = $outputpath.'RegulonDB/'.$data[$d].'_'.$bgOrder[$b].'.bg'; }

    #repeat for each nbrInstprior
    for(my $p = 0; $p < scalar(@nbrInstPrior); $p++)
    {
      $prior = $nbrInstPrior[$p];
      #repeat for each search method
      for(my $m = 0; $m < scalar(@search); $m++)
      {
        $method = $search[$m];
        if ($prior =~ m/^f0/ && $method == '0') 
        {#print "WARNING fixed prior not run for search 0 - SKIP\n"; 
         next;} 

        $newpath = $outputpath.'output/'.$data[$d].'/'.'bg_'.$bgOrder[$b].'/prior_'.$prior.'/method_'.$method;
        if (!(-e $newpath))
        { print FREP "Rerun all on $newpath\n"; next;}

        ##reset variables to compute average over 10 runs
        $convergence = 0;
        for (my$c=0;$c<scalar(@resume);$c++) { $resume[$c]= 0;}
        for (my$c=0;$c<scalar(@FC12);$c++) { $FC12[$c]= 0;}

        #now find the requested MS mtrxfile and FC txt/mtrx files
        for(my $r=1; $r <= $repeats; $r++)
        {
          $MSkolom = 'w'.$width.'_n'.$nbrMotifs.'_s'.$strand.'_x'.$overlap.'_M'.$maxInst.'_r'.$r;
          $textfile = $newpath.'/'.$MSkolom.'.txt';  $matrixfile = $newpath.'/'.$MSkolom.'.mtrx';   
          if (! (-e $matrixfile))
          { print FREP 'Run MS needed (not found): '. $matrixfile." - SKIP\n"; next; }
        
          ########## MS : pi=convergence by number of putative motifs
          $MSconv = &countmatrices($matrixfile);
          $text1 = $data[$d]."\t".$r."\t".$bgOrder[$b]."\t".$prior."\t".$method."\t".$MSconv;
          if ($MSconv <= 0)
          {
            print $text1;
            open(FOOF,">>$pifile");
            print FOOF $text1."\tend\n";
            close(FOOF);
          }
          else
          { #add to resume
            $convergence += $MSconv;
            $resume[0]++;            

            $FCkolom = '_FC_re';
            $textfile = $newpath.'/'.$MSkolom.$FCkolom.'.txt';  $matrixfile = $newpath.'/'.$MSkolom.$FCkolom.'.mtrx';
            if (! (-e $matrixfile))
            { print FREP 'Run FC needed (not found): '. $matrixfile." - SKIP\n"; next; }

            ############# FC :
            # for each motif : compute  
            # Occ(i) N(i) N(m) aIM minIM maxIM aMM minMM maxMM aIMpos aIMneg F PPV sens 
            # read CS, 
            # compute KL,BLiC tov regDBdb
 
            $nbrmotifs = &countmatrices($matrixfile);
            if ($nbrmotifs > 0) {$resume[1]++;}
            if ($nbrmotifs > 1){$resume[2]++;}

            #repeat for each motif
            for($rank = 1; $rank <=$nbrmotifs;$rank++)
            { 
              #store info in copytxt for verfication (temporary)
              $copytxt = $textfile.'FCsub'.$rank;
              open(FOFC,">$copytxt");

              #compute indicators from fc-txtfile for this motif
              ####################################################################
              $CS = 0; $F = 0; $PPV = 0; $sens = 0; $tp = 0; $fp = 0; $tfbsgo = 0; $motifsgo = 0;# reset
              $aIMpos = 0; $aIMneg =0; $aIM=0; $minIM=1000; $maxIM=-1;
              $OccW = 0; $aMM=0; $minMM=1000; $maxMM=-1;
              $nbrInst = 0; my $readrank = 0; $Nw = 0; $nbrMot = 0; $Ni = 0;
              my @motifships;
              open(FI, "<$textfile");
              while(<FI>)
              { 
                  $line = $_;
                  if ($line =~ m/^#id: box/) 
                  {
           #id: box_1_CS[18,GTACTmGTkwrCTrGTAC,1.44]_%InstanceCut=1_%MotifCut=0.86_nbrSeq=5_nbrInst=5_nbrMotifs=77
                    if ($line =~ m/^#id: box_(\d+)\_/) 
                    {
                      if (defined($1)){ $readrank = $1;}
                    }
                    else {print 'ERROR check script for reading! line = '.$line."-exit\n"; exit(-1);}
                    if ($readrank < $rank){next;}
                    if ($readrank > $rank){last;}
                    @split = split(/\,|\[|\]|\_/,$line);
                    if ( @split > 5) { $CS = $split[5];}
                    else { print 'ERROR check line '.$line.' in file '.$textfile."-exit\n"; exit(-1);}
                    if ($CS < 0)
                    { print 'ERROR check reading CS in line '.$line.' in file '.$textfile."-exit\n"; exit(-1);}
                    print FOFC $line;
                    $tfbsgo = 1; $motifsgo = 0; next;
                  }
                  if ($tfbsgo && ($line =~ m/^#CS/)){next;}
                  if ($tfbsgo && ($line =~ m/^#instance/))
                  { #instance memberships: (weighted_count=5, weighted_average_occurency=77)
                    chomp($line);
                    @split = split(/=/,$line);
                    $OccW = $split[2]; chop($OccW); #remove last bracket
                    $Ni = $split[1]; @split = split(/\,/, $Ni); $Ni = $split[0]; 
                    next;
                  }
                  if ($tfbsgo)
                  {
                    if ($line =~ m/^#motif/)
                    { #motif memberships: (weighted_count=70)
                      chomp($line);
                      @split = split(/=/,$line);
                      $Nw = $split[1]; chop($Nw); #remove last bracket
                      # all tfbs have been read for this motif, now read the motif memberships
                      $tfbsgo = 0;$motifsgo = 1;next;
                    } 
                    #read tfbs, occ and  membership
#format : emrK_60_77_+_AATCCTACAGGCGTAAGA_(shift=0,occ=55)	0.448
                    $nbrInst++; $start = -1; $weight = -1;
                    #extract membership
                    @split = split(/\t+|\s+/,$line);
                    if ( @split > 1) { $weight = $split[1];} 
                    else { print 'ERROR1 check line '.$line.' in file '.$textfile."-split=, @split ;exit\n"; exit(-1);}
                    #extract start/seq
                    @split = split(/\_/,$split[0]);

                    ##work around ERROR3 for eg LexA sequence molR_1 (the underscore should not be there in a seqname!)
                    if (@split == 7) 
                    { $split[1] = $split[0].$split[1]; shift(@split);}
                    if ( @split == 6) { $seq = $split[0];$start = $split[1]; }
                    else { print 'ERROR2 check line '.$line.' in file '.$textfile."-split=, @split ;exit\n"; exit(-1);}
                    #extract occ and remove last round hook
                    @split = split(/\=/,$split[5]);
                    if ($start == -1 || $weight == -1)
                    { print 'ERROR4 check reading line '.$line.' in file '.$textfile."-split=, @split ;exit\n"; exit(-1);}
                    ## process the instance
                    print FOFC $line;

                    $true = &validateTFBS($seq, $start, $width, $annothashref);
                    if($weight < $minIM){$minIM = $weight;}  if($weight > $maxIM){$maxIM = $weight;}
                    if ($true == 1) 
                    { $tp++; $aIMpos += $weight;} 
                    else 
                    { $fp++; $aIMneg += $weight; } 
                    $aIM += $weight; 
                  }
                  if ($motifsgo)
                  {
                    if ($line =~ m/^#/ || $line =~ m/^\n/) 
                    { 
                      last; # everything has been read for this motif
                    } 
                    #read motif membership and store
                    $nbrMot++; $weight = -1;
                    #extract membership
                    chomp($line);
                    @split = split(/\t/, $line);
                    if ( @split > 1) { $weight = $split[1];} 
                    else { print 'ERROR5 check line '.$line.' in file '.$textfile."-split=, @split ;exit\n"; exit(-1);}
                    if($weight < $minMM){$minMM = $weight;}  if($weight > $maxMM){$maxMM = $weight;}
                    #store
                    $aMM += $weight; 
                  }
              }
              close(FI);
              close(FOFC);
              # compute F,PPV,sens of the previously read tfbs
              $sens = $tp/$annotP;
              if ($tp+$fp > 0) { $PPV = $tp/($tp+$fp);} 
              if (($PPV+$sens) > 0){$F = 2*$PPV*$sens/($PPV+$sens);}
              if ($nbrInst>0){$aIM /= $nbrInst; }
              if ($nbrMot>0){$aMM /= $nbrMot; }
              if ($tp>0){$aIMpos/=$tp;}if ($fp>0){$aIMneg/=$fp;}
              # store info to be printed
#print FO "TF\trun\tbg\tprior\tsearch\tMSconv\tFCtrk\tOccW\tNw\taIM\tminIM\tmaxIM\taMM\tminMM\tmaxMM\taIMpos\taIMneg\tF\tPPV\tsens\tCS\tDBid\tKL\tpBLiC\n";
              $text = $text1."\tFC_".'a_'.$rank."\t".$OccW."\t".$Ni."\t".$Nw."\t".$aIM."\t".$minIM."\t".$maxIM."\t".$aMM."\t".$minMM."\t".$maxMM."\t".$aIMpos."\t".$aIMneg."\t".$F."\t".$PPV."\t".$sens."\t".$CS;

              $perf = $OccW."\t".$Ni."\t".$Nw."\t".$aIM."\t".$minIM."\t".$maxIM."\t".$aMM."\t".$minMM."\t".$maxMM."\t".$aIMpos."\t".$aIMneg."\t".$F."\t".$PPV."\t".$sens."\t".$CS;
                # add to resume         @FCb12 = (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0); #F,PPv,sens,CS,aW,aW+,aW-,minW,maxW,aOcc,aOcc+,aOcc-,minOcc,maxOcc,ML012:F,PPV,sens,nbrInst
              if ($rank <= 2)
              {
                @values = split(/\t/,$perf);
                for (my$g=0;$g<15;$g++)
                { $FC12[15*($rank-1)+$g] += $values[$g];}
              }

              #copy this motif PWM into a separate file, to be deleted afterwards
              $copymtrx = &copymotif($matrixfile, $rank);

              #run MotifComparison for this FC-motif against regDB database and write results to pi-file
              #########################################################################################
              &motifcomparison($copymtrx, $pifile, $width, $bgfile, $text);
              #open(FOT, ">>$pifile");
              #print FOT $text."\n";
              #close(FOT);

              ##delete copymatrix and copytext
              $command = 'rm -f '.$copymtrx; system($command);
              $command = 'rm -f '.$copytxt; system($command);

            } #end for this ranked motif
          } # if conv > 0
          # report progress 
          ##if ($r == $repeats){ print $data[$d]."-".$bgOrder[$b]."-".$prior."-".$method."-done\n";}   
        } #end of this repeat


        ##average over 10 runs and print to pi-file
        if ($resume[0] > 0){$convergence /= $resume[0];}
        if ($resume[1] > 0){ for (my$c=0;$c<15;$c++){ $FC12[$c] /= $resume[1];}}
        if ($resume[2] > 0){ for (my$c=15;$c<30;$c++){ $FC12[$c] /= $resume[2];}}

        #print
        open(FOO,">>$pifile");
        my $value = $resume[0];
        $text1 = $data[$d]."\tavg$value\t".$bgOrder[$b]."\t".$prior."\t".$method."\t".$convergence;
        ### perf result with FC motifs
        for(my$rnk=1;$rnk <= 2;$rnk++)
        {
            if ($rnk == 1){$value = $resume[1];} else {$value = $resume[2];}
            if ($value > 0)
            {
              $text1 = $data[$d]."\tavg$value\t".$bgOrder[$b]."\t".$prior."\t".$method."\t".$convergence;
              $text = $text1."\tFC_a_".$rnk;
              for (my$g=0;$g<15;$g++)
              { $text .= "\t".$FC12[15*($rnk-1)+$g];}
              print FOO $text."\n"; 
            }
        }    
        close(FOO);
      } #end of this search
    } #end of this prior
  } #end of this bgmodel
  print 'INFO @data[$d] = '.$data[$d]." - FINISHED\n";  
} #end of this regulator
close(FREP);
print 'END script finishes succesfully'."\n";         
           
######################################## SUBROUTINES
######################################################################################
# local time in format 0000-00-00 00:00:00
######################################################################################
sub time
{ 
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,
      $yday,$isdst)=localtime(time);
  my $format = sprintf ("%4d-%02d-%02d %02d:%02d:%02d",$year+1900, $mon+1,$mday,$hour,$min,$sec); 
  return($format);
}
######################################################################################
#loading variables from file into main
######################################################################################
sub readsettings 
{ 
  my ($file) = @_;
  open(FI, $file);
  while(<FI>) 
  {
    my $line = $_; 
    chomp $line;
    if ($line =~ m/^\$/ || $line =~ m/^\@/){ eval($line);}
  }
  close(FI);
}

######################################################################################
#create annotated hash based on annotatie-file contents
## changed for 250_50_new, now containes start AND end information (as length can be variable)
## we assume that the file only has ONE motif with its TFBS's
###################################################################################### 
sub annotatedhash # my ($annotP, $hashref)= &annotatedhash($annotatie);
{
  my ($annotfile) = @_; 

  open(FI1, $annotfile);
  my %hash; 
  my $annotNr = 0; 
  my $seqid; my $start, my $end; 
  while (<FI1>)
  {
    my $line = $_;
    chomp($line);
    if ($line =~ m/^\#/){next;}
    # read the site-starts !!!! and site ends !!!! for this motif 
    #format (urv pos 202 [fwd] L 17 end 218)
    my @site = split(/\s+/,$line);
    if (@site > 7) ##
    { $seqid = $site[0]; $start = $site[2]; $end = $site[7]; ##
      if (exists $hash{$seqid}){ push(@{$hash{$seqid}},$start); push(@{$hash{$seqid}},$end);}
      else { $hash{$seqid} = ["$start"]; push(@{$hash{$seqid}},$end);} 
      if ($start > -1) {$annotNr++;} #track number of positive annotated sites
    }
  }
  close(FI1);
  return ($annotNr, \%hash);
}

######################################################################################
#count the number of matrices in a matrixfile
######################################################################################
sub countmatrices # ($D1) = &countmatrices($matrixfile);
{
  my ($fName) = @_; 
  open(FI, "<$fName");
  my $count = 0; my $motif = ''; my @split;
  while(<FI>)
  { 
    my $line = $_;
    if ($line =~ m/^#ID/) #line of motif #ID = box_1_1_TCGATAACnTGTT_[PWM=CS-based]
    { # for fc, both CS as W may be reported : count this only once ! %2009/12/02
      @split = split("PWM",$line);
      if (@split >1) #this means it is fc-output
      {
        if ( $split[0] eq $motif) {next;} #do not count the second W-motif 
        $motif = $split[0];
      }
      $count++;
    }
  } 
  close(FI); 
  return($count);
}
######################################################################################
# extract from MStxtfile those motifs that belong to MRtxtfile, in the correct ranked order
######################################################################################
sub gettfbsinfo #$txt2file = &gettfbsinfo($txtfile);
{
  my ($MRfile) = @_; 
  my @splitt= split("_MR", $MRfile); my $MSfile = $splitt[0].'.txt';
  if (!(-e $MSfile))
  { print 'WARNING '.$MSfile." : file not found - DEBUG sub gettfbsinfo - exit!\n"; exit(-1);}

  my $txt2 = $MRfile.'MRsub';

  my @split; my $motifid; my $id; my $go = 0;

  open(FOUT, ">$txt2");
  print FOUT '#INCLUSive GFF File'."\n#\n";

  # read through MRfile
  open(FI, "<$MRfile");
  while(<FI>)
  { 
    my $line = $_;
    if ($line =~ m/^\t/ || $line =~ m/^\#/) {next;}
    if ($line =~ m/^\d+\)/) #line of ranked motif 1)box_31_1_wkwACTGTATATnnAynCAG	126.981
    { @split = split(/\)|\s+|\t+/,$line);
      $motifid = $split[1]; 
      # find this motif in MStextfile and copy all tfbs lines
      $go = 0;
      open(FI2, "<$MSfile");
      while(<FI2>)
      { 
        my $linee = $_;
        if ($linee =~ m/^\#id/) 
        { #(#id: box_31_1_wkwACTGTATATnnAynCAG	cons...)
          @split = split(/\t+|\s+/,$linee);
          $id = $split[1];
          if ($id eq $motifid) {$go = 1;} elsif ($go == 1){ last;}
        }
        if ($go) #print instances
        { print FOUT $linee;}   
      }
      close(FI2);
      if (!$go)
      { print 'WARNING '.$motifid.' : not found in '.$MSfile." - DEBUG sub gettfbsinfo - exit!\n"; exit(-1);}
    }
  } 
  close(FI);
  close(FOUT);
 
  return($txt2); 
}
######################################################################################
# check if an instance overlaps with an annotated TFBS from $refhash
###################################################################################### 
sub validateTFBS #$true = &validateTFBS($seq, $start, $width, $annothashref);
{
   ## compare with ($refhash) from annotatie-file 
   my ($seqq, $startt, $ww, $reffhash) = @_;
   my $true = 0; my $Tstart; my $Tend;
   if (exists $$reffhash{$seqq})
   { 
     for my $key ( keys %$reffhash ) 
     { if ($seqq eq $key) 
       { for(my $s=0; $s < scalar(@{ $$reffhash{$seqq} }); $s++)
         { $Tstart = $$reffhash{$seqq}[$s]; $s++;
           $Tend = $$reffhash{$seqq}[$s]; 
           if ($Tstart < 0) { last;} #this sequence has no annotated sites
           if ( ($startt > ($Tstart - int($ww/2))) && ($startt < ($Tend - int($ww/2) )))
           {$true = 1; last;}
         }
       }
     }
   }
   return($true);
}

######################################################################################
#copy rank-motif from a file into a separate file, applicable for TFBS for MR and PWM for MR/FC
######################################################################################
sub copymotif # $copymotif = &copymotif($mtrxfiles[1], $rank);
{
  my($file, $rk) = @_;

  #get the requested (rk) motif from file :make a copy of all info untill next motif to file.sub 
  ## for FC-MATRIX-output, only consider the CS-motif in case also W-motif is printed 
  my $copy = $file.'sub'.$rk;
  open(FOM, ">$copy");
  print FOM "#INCLUSive Motif Model\n#\n";
  open(FIM, "<$file");
  my $rkk = 0; my $go = 0; my @spl; my $print = 0;
  while(<FIM>)
  { 
    my $lin = $_;
    if ($lin =~ m/^#ID/ || $lin =~ m/^#id/) #line of motif 
#format MR-tfbs : #id: box_31_1_wkwACTGTATATnnAynCAG	cons...
#format FC-mtrx : #ID = box_1_CS[...]_W[...]_E=0.329804_nbrInst=8_[PWM=CS-based] or _[PWM=width-based]
#format MR-mtrx : #ID = box_31_1_wkwACTGTATATnnAynCAG
    { # for fc, both CS as W may be reported : count and copy only CS-based (printed first)
      @spl = split("PWM=",$lin);
      if (@spl > 1) #this means it is double FC-output
      { if ( $spl[1] =~ m/CS/ ) {$rkk++;}
        else { $go = 0; if ($rkk == $rk){last;}} #CS-motif is already done, skip W representation.
      } # this is MR or single FC output
      else {$rkk++;} 
      # get the right motif
      if ($rkk < $rk) {next;}
      elsif ($rkk == $rk) { $go = 1;}
      else {last;}
    }
    if ($go) { print FOM $lin; $print = 1;} #copy each line
  } 
  close(FIM); 
  close(FOM);
  if (!$print)
  { print 'WARNING '.$copy." : copyfile does not contain a motif - DEBUG sub copymatrix - exit!\n"; exit(-1);}
  return($copy);
}

######################################################################################
#compute performance indicators  from a file with a list of instances for ONE motif !!
######################################################################################
sub indicators ## $indicators = &indicators($tfbsfile, $width, $annothashref,$annotP);
{
  my ($motif, $w, $hashref,$annotPP) = @_;
  my $tp = 0; my $fp = 0; my $F= 0;my $PPV= 0; my $sens = 0; my$nbrInst = 0;
  my @split;
  my $seq; my $start; my $true;
  my $CS = 0; my $LL = 0;

  my $ml = 0;
  open(FIML, "<$motif");
  while(<FIML>)
  {
    my $line = $_;
    if ($line =~ m/^\#MotifSampler results summary/) {last;} 
    if ($line =~ m/^\#id/) # for the case of MR-reading tfbs file
    { 
#format : (#id: box_1_1_TCGATAACnTGTT consensus: TCGATAACnTGTT sequences: 10 instances: 10 cs: 1.55 ic: 1.55 ll: 152.32)
      chomp($line);
      # extract CS and LL of this motif
      @split = split(/\t+|\s+/,$line);
      if (@split > 9){ $CS = $split[9];} #cs
      if (@split > 13){ $LL = $split[13];} #ll
    }
    if ($line =~ m/MotifLocator/ || $line =~ m/MotifSampler/) 
    { 
      if ($line =~ m/MotifLocator/){$ml = 1;}
      $nbrInst++;
#format : (SEQ0 MotifLocator misc_feature 420 432 3.02613e+06 + . id "box_1_1_TCGATAACnTGTT"; site "TCGATAACCTGTT";)
      @split = split(/\t+|\s+/,$line);
      if ( @split > 5) { $seq = $split[0];$start = $split[3]; }
      else { print 'ERROR sub indicators check line '.$line.' in file '.$motif."-exit\n"; exit(-1);}
      $true = &validateTFBS($seq, $start, $w, $hashref);
      if ($true == 1) { $tp++;} else { $fp++; }
    }
  }
  close(FIML);
  # compute F,PPV,sens of the previously read tfbs 
  $sens = $tp/$annotPP;
  if ($tp+$fp > 0) { $PPV = $tp/($tp+$fp);} 
  if (($PPV+$sens) > 0){$F = 2*$PPV*$sens/($PPV+$sens);}

  my $indicators = $F."\t".$PPV."\t".$sens;
  if ($ml) {$indicators = $indicators."\t".$nbrInst;} #motifranking
  if ($CS != -1){$indicators = $indicators."\t".$CS;}
  if ($LL != -1){$indicators = $indicators."\t".$LL;}
  return($indicators);
}


######################################################################################
#run motifcomparison with KL or BLiC for one motif, print +similarity results to file
######################################################################################
sub motifcomparison ## &motifcomparison($copymtrx, $pifile, $width, $bgfile, $text);
{
  my($matrices2, $druk, $w, $b, $txt) = @_; 
  my$s = $w; #shift
  my$x = int($w/2); #overlap
  my $regDB = $outputpath.'RegulonDB_freq_6-4.mtrx'; #path to independently created full regDB  

  ## run the MC commando KL
  my $KLfile = $matrices2.'KL';
if (!(-e $KLfile))
{
  my $command = $driverpath.'applications/MotifComparison -m '.$matrices2.' -o '.$KLfile.' -d '.$regDB.' -t 0.4 -s '.$s.' -x '.$x.' -l 0';
  system($command);
}
  ## run the MC commando BLiC
  my $BLiCfile = $matrices2.'BLiC';
if (!(-e $BLiCfile))
{
  $command = $driverpath.'applications/MotifComparison -m '.$matrices2.' -o '.$BLiCfile.' -d '.$regDB.' -t 0.001 -s '.$s.' -x '.$x.' -l 1 -n '.$w.' -b '.$b; #no entry for -p (Dirichlet default)
  system($command);
}

  ## write similarities to $pifile
#KLformat : #query:query_consensus	match:match_consensus	score;(shiftm-d,shiftd-m)
#box_1_1_ATTCCTACnnnTGTArGA:ATTCCTACnnnTGTArGA	box_ann1_ps0.001_TTCCTACAsyTGTAAGAAw:TTCCTACAsyTGTAAGAAw	0.270935;(-1,1)
#BLiCformat : query:query_consensus	match:match_consensus	score;shift;overlap;strand	p-value;common;(n:avg,stdev)
#box_1_1_ATTCCTACnnnTGTArGA:ATTCCTACnnnTGTArGA	box_ann1_ps0.001_TTCCTACAsyTGTAAGAAw:TTCCTACAsyTGTAAGAAw	20.5;-1;17;fwd	5.3301e-07;0;(n=18:9.07,1.06)
my $print = 0;
  open(INPUT1, "<$KLfile"); 
  open(INPUT2, "<$BLiCfile"); 
  open(FOOT,">>$pifile");
  my $line1; my $line2; my @result1; my $regid; my @result2; my $KL; my $pBLiC; my @spl;
  while ( <INPUT1>) 
  {
    $line1 = $_;
#print "line 1 = $line1";
    $line2 = <INPUT2>; #read the same line in other file
#print "line 2 = $line2";
    #if ($line1 =~ m/^\#/) {next;} # masked lines for distant matrices / also read incorrect matrices to track topKL/topBLiC.
    if ($line1 =~ m/box/) # this is a similairty line (correct or incorrect)
    {
      if ( !($line1 =~ m/^\#/ && $line2 =~ m/^\#/)) # one or both show similarity 
      { 
        @result1 = split(/\t/, $line1);
        @result2 = split(/\t/, $line2);
        @spl = split(/\:/, $result1[1]); $regid = $spl[0]; @spl = split(/\:/, $result2[1]);
        if ($regid ne  $spl[0]) {print 'ERROR - sub MCevalutation : error in file-reading1 - debug $regidKL = '.$regid.' - regidBLiC = '.$spl[0].' - exit'."\n"; exit(-1);}
        # get KL score
        @spl = split(/\;/,$result1[2]); $KL = $spl[0];
        # get pvalue BLiC score
        @spl = split(/\;/,$result2[3]); $pBLiC = $spl[0];
        # print to perf file
        print FOOT $txt."\t".'DB'.$regid."\t".$KL."\t".$pBLiC."\n";  
        $print = 1;  
      }
      # go to next line in both files 
    }
  }
  close(INPUT1);
  close(INPUT2);

  if (!$print){ print FOOT $txt."\tDB-\tKL\tBLiC-\n";  }

  close(FOOT);
  return;
}

######################################################################################
#run motiflocator with different thresholds (0.8, 0.85, 0.9), print results to file
######################################################################################
sub motiflocator ## &motiflocator($copymtrx,$data[$d], $bgfile);
{
  my($matrices2, $datai, $b) = @_;
  my @spl;
  ## get the fastafile with sequences
  my $fastafile = $fastapath.$datai.'.fna';
  if (!(-e $fastafile))
  { print 'WARNING '.$fastafile." : fastafile not found - ERROR\n"; exit(-1);}
                    
  ## copy the file so that the headerline only contains seq-id information
  my $fastafile1 = $outputpath.$datai.'.fnacopy';
  open(FREAD, "<$fastafile");
  open(FWRITE, ">$fastafile1");
  while(<FREAD>)
  {
    my $line = $_;
    if ($line =~ m/^\>/) # > |89_GCTGCGCTTATCGACAGTTA|180_CCTGGCTTTCAGGGCAGCGT/20/phr
    { chomp($line);
      @spl = split(/\//,$line);
      print FWRITE '>'.$spl[2]."\n";
    }
    else { print FWRITE $line;}
  }
  close(FREAD); 
  close(FWRITE);
 
  # run the algorithm for different tresholds
  my @results = ('','','');
  my @tresholds = (0.8,0.85,0.9);
  my $outputfile;
  for(my $tr = 0; $tr < scalar(@tresholds); $tr++) 
  { 
    $outputfile = $matrices2.'MLt'.$tr; # avoid dots in extension -> MLt0, MLt1, MLt2 
if (!(-e $outputfile))
{
    $command = $driverpath.'applications/MotifLocator -f '.$fastafile1.
                   ' -b '.$b.' -o '.$outputfile.' -m '.$matrices2.' -s 0 -t '.$tresholds[$tr].' -a 0';
    system($command);
}
    $results[$tr] = $outputfile;
  }
  #remove the $fastafile-copy
  $command = 'rm -f '.$fastafile1; system($command);

  return (@results);
}
