#!/usr/bin/perl -w
use strict;
use DBI;

##for local/server use, change path 
#my $driverpath = '/home/marleen/';
my $driverpath = '/users/sista/mclaeys/INCLUSive_for_users/';
## inclusive version 
$driverpath .= 'INCLUSive-3.1.5/';

## Date of last changes : 10 juni 2010
## !! FuzzyClustering : has been run with -p 0.3 instead of 0.05 and -m 0.5 instead of -1

# script to run ML-3.1.5
# output goes to subdirectory /users/sista/mclaeys/INCLUSive_tests/MS-250_50_new

# perl command for this script :
#################################################### 
# perl runML-250_50_new.pl $ARGV[0]
####################################################
# $ARGV[i] = file with settings for different @data, @bgorder/@prior/@search for each TF

####################################################
# subroutines
#################################################### 
# &readsettings 
####################################################

###########################################################################################
my $database = "motiefdetectie";
my $hostname = "dbserv.esat.kuleuven.be";
my $port = "3306";
my $username = "motiefdetectie";
my $password = 'gHte!ens';
my $dsn = "DBI:mysql:database=$database;host=$hostname;port=$port";
my $dbh;
$dbh = DBI->connect($dsn, $username, $password) or die("Could not connect to DBI!");
#my $dbh = DBI->connect("DBI:mysql:dbserv.esat.kuleuven.be:motiefdetectie", 'motiefdetectie', 'ste5OLic');
my $table = 'MotifLocator_INCLUSivetestsMS25050new';
my $now;

#create table MotifLocator_INCLUSivetestsMS25050new (DateTime DATETIME, TF VARCHAR(5), TFBSmax int(1), bgm VARCHAR(8), prior VARCHAR(20), search INT(1), run VARCHAR(20), MSdesc VARCHAR(150), DB_nbrInst decimal(4,1), DB_F decimal(4,3), DB_pvv decimal(4,3), DB_sens decimal(4,3), MRdesc VARCHAR(150), ML_MR_repeat int(2), topMR_nbrInst decimal(4,1),topMR_F decimal(4,3), topMR_pvv decimal(4,3), topMR_sens decimal(4,3), FCdesc VARCHAR(150), ML_FC_repeat int(2), topFC_nbrInst decimal(4,1),topFC_F decimal(4,3), topFC_pvv decimal(4,3), topFC_sens decimal(4,3), primary key (DateTime,TF,bgm,prior,search,run));

###########################################################################################

my @resume = (0,0,0,0,0,0,0,0,0,0); #repeat/nbrinst/F/pvv/sens voor MR en FC
my $run;

# fasta-path
my $fastapath;
my $outputpath;
if ( $driverpath =~ m/sista/) 
{ $fastapath = '/users/sista/vstorms/Datasets/Marleen_MotifSuite/250_50_new/';          
  $outputpath = '/users/sista/mclaeys/INCLUSive_tests/MS-250_50_new/';
}
else  
{ $fastapath = '/home/marleen/MS-250_50_new/RegulonDB/';
  $outputpath = $fastapath;
}

# variables to be loaded from the settingsfile
my @data; #= ('LexA');
my @bgOrder; # = ('Ecoli_2',2);
my @nbrInstPrior; #one for each TF (individual listing !)
my @search; #one for each TF (individual listing !)
my $repeats; #10

# for MR/FC file reconstruction
my $width; #to be read from the specific inputfile wM.txt
my $nbrMotifs = 1; 
my $strand = 0;
my $overlap = '-'; # not applicable 
my $maxInst; #to be read from the specific inputfile
my $MSdesc;
my $FCdesc = 'FC_tbasic_p03_m05';
my $MRdesc;
my $ovlap;
my $shift;

# parametersettings for ML : l/s/t/a : see text
 
# read the ARGV 
my $settingsfile = $ARGV[0];
# load the file
if (!(-e $settingsfile))
{ print 'ERROR '.$settingsfile." : file not found - exit\n"; exit(-1);}

# local variables
my $command;
my $fastafile; my $fastafile1;
my $datai;
my $outputdir;
my $bgfile;
my $selectfile;
my @inputmotifs = ('regDB','topMR','topFC');
my $Bgm;
my $Prior; 
my $Search;
my $outputdirsub;
my @split;
my $motif;
my $matrixfile;
my $outputfile;
my $wMfile;

my $annotatie;
my $tp; my $fp; my $f; my$pvv; my $sens; my $nbrInst; my $nbrRepeats;


# update the parameters as read from the settingsfile (@data/@bgorder/@prior/@search)
&readsettings($settingsfile); 

if (scalar(@data) != scalar(@nbrInstPrior) || scalar(@data) != scalar(@search) || scalar(@data) != scalar(@bgOrder))
{print 'ERROR '.$settingsfile." : scalars = ".scalar(@data).'/'.scalar(@bgOrder).'/'.scalar(@nbrInstPrior).'/'.scalar(@search)." - individual listing of prior/search per TF ! - exit\n"; exit(-1);}

# repeat for each dataset(=regulator)
for(my $d = 0; $d < scalar(@data); $d++)
{
  $datai = $data[$d];
  $now = &time;
  # get the motifwidth and maxInst (the wM.txt file should be available)
  $wMfile = $outputpath.'RegulonDB/'.$datai.'_wM.txt';
  if (!(-e $wMfile))
  { print 'WARNING '.$wMfile." : wMfile not found - exit\n"; exit(-1);}
  &readsettings($wMfile);
  if ( $width == 0 || $maxInst == 0)
  { print 'WARNING '.$wMfile.' : wrong $width/$maxInst format - exit'."\n"; exit(-1);} 
  $MSdesc = 'w'.$width.'_n'.$nbrMotifs.'_s'.$strand.'_x'.$overlap.'_M'.$maxInst;
  #set MRoverlap as half of $width
  $ovlap = int($width/2);
  $shift = $width;
  $MRdesc = 'MR_s'.$shift.'_x'.$ovlap.'_r5_m0_t04'; ####################keep r/m/t fixed as it has been done for now...#################

  # get the fastafile with sequences
  $fastafile = $fastapath.$datai.'.fna';
  if (!(-e $fastafile))
  { print 'WARNING '.$fastafile." : fastafile not found - SKIP\n"; next;}
  print 'Processing '.$fastafile."...\n"; 
  ## copy the file so that the headerline only contains seq-id information
  $fastafile1 = $outputpath.$datai.'.fnacopy';
  open(FREAD, "<$fastafile");
  open(FWRITE, ">$fastafile1");
  while(<FREAD>)
  {
    my $line = $_;
    if ($line =~ m/^\>/) # > |89_GCTGCGCTTATCGACAGTTA|180_CCTGGCTTTCAGGGCAGCGT/20/phr
    { chomp($line);
      @split = split(/\//,$line);
      print FWRITE '>'.$split[2]."\n";
    }
    else { print FWRITE $line;}
  }
  close(FREAD); close(FWRITE);

  # create regulator subdirectory for output
  $outputdir = $outputpath.'output/'.$datai.'/';
  #if (!(-e $outputdir))
  #{ $command = 'mkdir -p '.$outputdir; system($command);}
  
  # get the backgroundfile (these should be available)
  $Bgm = $bgOrder[$d];
  if ($Bgm =~ m/Ecoli/) { $bgfile = $outputpath.'RegulonDB/'.$Bgm.'.bg';}
  else { $bgfile = $outputpath.'RegulonDB/'.$datai.'_'.$Bgm.'.bg';}
  if (!(-e $bgfile))
  { print 'WARNING '.$bgfile." : backgroundfile not found - exit\n"; exit(-1);} 
  # get the nbrInstprior (for directory and $matrixfile retrieval)
  $Prior = $nbrInstPrior[$d];
  $Search = $search[$d];

  # go to output-subdirectory
  $outputdirsub = $outputdir.'bg_'.$Bgm.'/';

  #if (!(-e $outputdirsub))
  #{ $command = 'mkdir -p '.$outputdirsub; system($command); }


  # for evaluation 
  # create a hash of arrays with results from annotatie-file for this TF
  $annotatie =  $outputpath.'RegulonDB/'.$datai.'_annotatie.txt';
  if (!(-e $annotatie))
  { print 'WARNING '.$annotatie." : annotatie file not found - SKIP further performance computation !\n"; next;}
  my ($annotP, $annothashref) = &annotatedhash($annotatie);

  # for resume
  for(my $res=0; $res < 10; $res++){$resume[$res]=0;} ##reset

  # repeat for different 'source' motif
  for(my $so = 0; $so < scalar(@inputmotifs); $so++) # $so = 0 => $inputmotifs[$so] = /regDB/
  {  
    if ($so == 0){ $repeats = 1;} else { $repeats = 10; $outputdirsub = $outputdir.'bg_'.$Bgm.'/prior_'.$Prior.'/method_'.$Search.'/';}

    for(my $r=1; $r <= $repeats; $r++)
    {
      if ($so == 0) 
      { $matrixfile = $outputpath.'RegulonDB/'.$datai.'_pwm_ps0.001.mtrx';
        $motif = $inputmotifs[$so].'ps0.001'; #'regDB';
        $run = 'resume';
      }
      elsif ($inputmotifs[$so] =~ m/topMR/) 
      { $matrixfile = $outputdirsub.$MSdesc.'_r'.$r.'_'.$MRdesc.'.mtrx';
        $motif = $inputmotifs[$so].'r'.$r;
        $run = 'MRrun'.$r;
      }
      elsif ($inputmotifs[$so] =~ m/topFC/)
      { $matrixfile = $outputdirsub.$MSdesc.'_r'.$r.'_'.$FCdesc.'.mtrx';
        $motif = $inputmotifs[$so].'r'.$r;
        $run = 'FCrun'.$r;
      } 
      if (!(-e $matrixfile))
      { print 'ERROR '.$matrixfile." : motif-file not found - exit\n"; exit(-1);}
      if ($so > 0)
      {($selectfile) = &getfirstmotif($matrixfile);}
      else {$selectfile = $matrixfile;}

      #check if there is a valid matrix
      if (!(-e $selectfile))
      { print 'WARNING : no matrix for '.$matrixfile.'- skip MotifLocator'."\n"; next;}

      # and run the algorithm 
      $outputfile = $outputdirsub.'ML_m'.$motif.'_s0_t085_a0.txt';
      $command = $driverpath.'applications/MotifLocator -f '.$fastafile1.
                   ' -b '.$bgfile.' -o '.$outputfile.' -m '.$selectfile.' -s 0 -t 0.85 -a 0';
      system($command);
    
      #remove the $selectfile-copy
      if ($so > 0 && -e $selectfile)
      { $command = 'rm -f '.$selectfile; system($command);}

      #compute the F/pvv/sens of the retrieved TFBS's in $outputfile using the annotated hash ref
      #code based on runResume-250_50_new_rev5.pl
      open(FI, "<$outputfile");
      $tp = 0; $fp = 0; $f=0; $pvv=0; $sens = 0;
      $nbrInst = 0;
      while(<FI>)
      {
        my $lin = $_;
        if ($lin =~ m/MotifLocator/) 
        { $nbrInst++;
          my ($truee) = &validateTFBS($lin, $width, $annothashref);
          if ($truee == 1) { $tp++;}
          elsif ($truee == 0) {$fp++;} 
          else { print 'ERROR check line '.$lin.' in file '.$outputfile."-exit\n"; exit(-1);}
        }
      }
      $sens = $tp/$annotP;
      if ($tp+$fp > 0) { $pvv = $tp/($tp+$fp);} 
      if (($pvv+$sens) > 0){$f = 2*$pvv*$sens/($pvv+$sens);}      
      close(FI); 

      #insert DB (only once $r=1) MR/FC individual runs into DBI table and add to resume (MR/FC)
      if ($so == 0)
      {
        $command = "insert into $table (DateTime, TF, TFBSmax, bgm, prior, search, run, MSdesc, DB_nbrInst, DB_F, DB_pvv, DB_sens, MRdesc, FCdesc) values ('$now','$datai','$maxInst' ,'$Bgm', '$Prior', $Search, '$run', '$MSdesc', $nbrInst, $f, $pvv, $sens, '$MRdesc', '$FCdesc')";
      }
      elsif ($so == 1 ) #MR : also add results to resume
      {
        $command = "insert into $table (DateTime, TF, TFBSmax, bgm, prior, search, run, MSdesc, MRdesc, topMR_nbrInst,topMR_F, topMR_pvv , topMR_sens) values ('$now','$datai','$maxInst' ,'$Bgm', '$Prior', $Search, '$run', '$MSdesc', '$MRdesc', $nbrInst, $f, $pvv, $sens)";
        $resume[0]++; $resume[1] += $nbrInst; $resume[2] += $f; $resume[3] += $pvv; $resume[4] += $sens;     
      }
      elsif ($so == 2 ) #FC : also add results to resume
      {
        $command = "insert into $table (DateTime, TF, TFBSmax, bgm, prior, search, run, MSdesc, FCdesc, topFC_nbrInst,topFC_F, topFC_pvv , topFC_sens) values ('$now','$datai','$maxInst' ,'$Bgm', '$Prior', $Search, '$run', '$MSdesc', '$FCdesc', $nbrInst, $f, $pvv, $sens)";
        $resume[5]++; $resume[6] += $nbrInst; $resume[7] += $f; $resume[8] += $pvv; $resume[9] += $sens; 
      }
      $dbh->do("$command");
    
    } # end of repeats

    print 'INFO @data[$d] = '.$data[$d]."/ $motif - FINISHED\n";
  } #end of DB/MR/FC

  #now also add resume results of MR and FC to regDB line
  #normalize
  if ($resume[0] > 0) 
  { for(my $res = 1; $res < 5; $res++){$resume[$res] /= $resume[0];}} #MR
  $nbrRepeats = $resume[0]; $nbrInst = $resume[1]; $f = $resume[2]; $pvv = $resume[3]; $sens = $resume[4];
  $command = "update $table set ML_MR_repeat = $nbrRepeats, topMR_nbrInst = $nbrInst,topMR_F  = $f,topMR_pvv  = $pvv, topMR_sens = $sens where DateTime = '$now' and TF = '$datai' and bgm = '$Bgm' and prior = '$Prior' and search = '$Search' and run = 'resume'";
  $dbh->do("$command");
  if ($resume[5] > 0) 
  {for(my $res = 6; $res < 10; $res++){$resume[$res] /= $resume[5];}
  } #FC
  $nbrRepeats = $resume[5]; $nbrInst = $resume[6]; $f = $resume[7]; $pvv = $resume[8]; $sens = $resume[9];
  $command = "update $table set ML_FC_repeat = $nbrRepeats, topFC_nbrInst = $nbrInst,topFC_F  = $f,topFC_pvv  = $pvv, topFC_sens = $sens where DateTime = '$now' and TF = '$datai' and bgm = '$Bgm' and prior = '$Prior' and search = '$Search' and run = 'resume'";
  $dbh->do("$command");
 
  #remove the $fastafile-copy
  $command = 'rm -f '.$fastafile1; system($command);

}  
print 'INFO script finishes succesfully'."\n";

######################################## SUBROUTINES
######################################################################################
# local time in format 0000-00-00 00:00:00
######################################################################################
sub time
{ 
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,
      $yday,$isdst)=localtime(time);
  my $format = sprintf ("%4d-%02d-%02d %02d:%02d:%02d",$year+1900, $mon+1,$mday,$hour,$min,$sec); 
  return($format);
}
######################################################################################
#loading variables from file into main
######################################################################################
sub readsettings 
{ 
  my ($file) = @_;
  open(FI, $file);
  while(<FI>) 
  {
    my $line = $_; 
    chomp $line;
    #print '$line = '.$line."\n";
    if ($line =~ m/^\$/ || $line =~ m/^\@/){ eval($line);}
  }
  close(FI);
}
######################################################################################
#copy the first motif from the matrixfile in another file
######################################################################################
sub getfirstmotif #($selectfile) = &getfirstmotif($matrixfile);
{
  my ($fName) = @_; 
  open(FI, "<$fName");
  my $newfile = $fName.'first';
  open(FO, ">$newfile");

  my $count = 0; 
  while(<FI>)
  { 
    my $line = $_;
    if ($line =~ m/^#ID/) #line of motif #ID = box_1_1_TCGATAACnTGTT_[PWM=CS-based]
    { $count++;}
    if ($count == 2) {last;} # stop at second motif
    print FO $line; #else copy line to newfile
  } 
  close(FI); 
  close(FO);
  if ($count == 0){ my $commandd = 'rm -f '.$newfile; system($commandd); $newfile = 'NULL';}

  return($newfile);
}


######################################################################################
###COPY FROM runResume-250_50_new_rev5.pl
#create annotated hash based on annotatie-file contents
## changed for 250_50_new, now containes start AND end information (as length can be variable)
## we assume that the file only has ONE motif with its TFBS's
###################################################################################### 
sub annotatedhash # my ($annotP, $hashref)= &annotatedhash($annotatie);
{
  my ($annotfile) = @_; 

  open(FI1, $annotfile);
  my %hash; 
  my $annotNr = 0; 
  my $seqid; my $start, my $end; 
  while (<FI1>)
  {
    my $line = $_;
    chomp($line);
    if ($line =~ m/^\#/){next;}
    # read the site-starts !!!! and site ends !!!! for this motif 
    #format (urv pos 202 [fwd] L 17 end 218)
    my @site = split(/\s+/,$line);
    if (@site > 7) ##
    { $seqid = $site[0]; $start = $site[2]; $end = $site[7]; ##
      if (exists $hash{$seqid}){ push(@{$hash{$seqid}},$start); push(@{$hash{$seqid}},$end);}
      else { $hash{$seqid} = ["$start"]; push(@{$hash{$seqid}},$end);} 
      if ($start > -1) {$annotNr++;} #track number of positive annotated sites
    }
  }
  close(FI1);
#print "annotatiefile = $annotfile; nbrAnnotSites = $annotNr\n";

  return ($annotNr, \%hash);
}


######################################################################################
###COPY FROM runResume-250_50_new_rev5.pl 
###################################################################################### 
sub validateTFBS #($truee, $aisss, $probocc) = &validateTFBS($line, $width, $refhash);
{ 
      my ($linee, $w, $reffhash) = @_;
#print 'sub validate TFBS line = '.$linee."\n";
      my $true = 0; 
      my $Tstart; my $Tend;

      # read the seq-id and site-start ## and site-END
      my @site = split(/\t+|\s+/,$linee);
      my @split;
      my $go = 1;
      my $seq = -1; my $start = -1; my $end = -1; 
      #(SEQ0 MotifSampler misc_feature 420 432 3.02613e+06 + . id "box_1_1_TCGATAACnTGTT"; site "TCGATAACCTGTT";)
      if ( @site > 5)
      { $seq = $site[0];$start = $site[3];$end = $site[4];} else {$go = 0;} #2009/12/02 reset in case of wrong format

      if (!$go) {$true = -1;} #2009/12/02 reset (stop for this motif) and read on to next motif
      else
      {
        ## now compare with ($refhash) from annotatie-file 
        $true = 0;
        if (exists $$reffhash{$seq})
        { 
          for my $key ( keys %$reffhash ) 
          { if ($seq eq $key) 
            { for(my $s=0; $s < scalar(@{ $$reffhash{$seq} }); $s++)
              { $Tstart = $$reffhash{$seq}[$s]; $s++;
                $Tend = $$reffhash{$seq}[$s]; 
                if ($Tstart < 0) { last;} #this sequence has no annotated sites
                if ( ($start > ($Tstart - int($w/2))) && ($start < ($Tend - int($w/2) )))
                {$true = 1; last;}
              }
            }
          }
        } 
      }
      return ($true);
}


