#!/usr/bin/perl -w
#use strict;

##for local/server use, change path 
#my $driverpath = '/home/marleen/';
my $driverpath = '/users/sista/mclaeys/INCLUSive_for_users/';
## inclusive version (MotifRanking)
$driverpath .= 'INCLUSive-3.1.5/';
my $command;

## Date of last changes : 8 dec 2010 -> naar output1
## 15 juni 2011 : alles in folder output2 !

# script to run MR-3.1.5 on data 250_50_new
# output goes to the output-directory where the inputfiles are available

# perl command for this script :
#################################################### 
# perl runMR-250_50_new.pl $ARGV[0] 
####################################################
# $ARGV[0] = refer to settingsfile from runMS-3.1.5 ->route to dir of outputfiles
# the parameters are also written in this file : -s, -x, -r, -m, -t

####################################################
# subroutines
#################################################### 
# &readsettings 
####################################################

# fasta-path
#my $fastapath;
my $outputpath;
if ( $driverpath =~ m/sista/) 
{ #$fastapath = '/users/sista/vstorms/Datasets/Valerie/250_50_new/';
  $outputpath = '/users/sista/mclaeys/INCLUSive_tests/MS-250_50_new/';
}
else  
{ #$fastapath = '/home/marleen/MS-250_50_new/';
  $outputpath = '/home/marleen/MS-250_50_new/';
}

# variables to be loaded from the settingsfile
my @data; #= ('LexA');
#my @space; # = ('coregulation/');
my @bgOrder; # = (0,1,2,3);
#my $bmatrix; # = 1
#my $btrack; # = 1;
my @nbrInstPrior; # = ('0.25_0.25','0.9_0.25', 'u', 'e0_1', 'e0_0_1', 'e0_0.45_0.3_0.25','f0_1');
my @search; # = (1); 
my $runs; # = 100;
my $repeats; #10
# removed from settingsfile : 
my $width =0; #to be read from the specific inputfile wM.txt
my $nbrMotifs = 1; 
my $overlap = '-'; # not applicable 
my $strand = 0;
my $maxInst = 0; #to be read from the specific inputfile

my @parMRrank = (); # = (2 or 5); #-r : for all data !! it will be only one value
#my @parMRshift = (); # not determing, to be set as $width !!
#my @parMRovl = (); #determing : to be set as half of $width !!
my @parMRmode = ();
my @parMRtresh = (); 

# read the ARGV 
my $settingsfile = $ARGV[0];
if (!(-e $settingsfile)) { print 'ERROR '.$settingsfile." : file not found - end\n"; exit(-1);} 

# read the settings to reconstruct MR-inputfiles and set the MR parameters
&readsettings($settingsfile);

# for MR :
my $rank= $parMRrank[0];
my $shift = 0; # to be set as $width !!
my $ovlap = 0; # to be set as half of $width !!
my $tresh = 0.4; # the general default 
if (scalar(@parMRtresh) == 1){ $tresh = $parMRtresh[0];}
my $mode = 0; 
if (scalar(@parMRtresh) == 1){ $mode = $parMRmode[0];}

# local variables
my $matrixfile;
my $outputfile;
my $textfile;

# repeat for each dataset
for(my $d = 0; $d < scalar(@data); $d++)
{
  # get the motifwidth and maxInst (the wM.txt file should be available)
  $wMfile = $outputpath.'RegulonDB/'.$data[$d].'_wM.txt';
  if (!(-e $wMfile))
  { print 'WARNING '.$wMfile." : wMfile not found - SKIP MotifRanking\n"; next;}
  &readsettings($wMfile);
  if ( $width == 0 || $maxInst == 0)
  { print 'WARNING '.$wMfile.' : wrong $width/$maxInst format - SKIP MotifRanking'."\n"; next;} 
  #set MRoverlap as half of $width
  $ovlap = int($width/2);
  $shift = $width;

  # repeat for each backgroundfile
  for(my $b = 0; $b < scalar(@bgOrder); $b++)
  {
    $bgfile = $outputpath.'RegulonDB/'.$data[$d].'_'.$bgOrder[$b].'.bg';
    if ($mode == 2 && !(-e $bgfile))
    { print 'WARNING '.$bgfile." : backgroundfile not found (MRmode = IC)  - SKIP\n"; next;} 

    #repeat for each nbrInstprior
    for(my $p = 0; $p < scalar(@nbrInstPrior); $p++)
    {
      $prior = $nbrInstPrior[$p];
      #repeat for each search method
      for(my $m = 0; $m < scalar(@search); $m++)
      {
        $method = $search[$m];
        if (!(-e $outputpath.'output2/'.$data[$d].'/'.'bg_'.$bgOrder[$b].'/prior_'.$prior.'/method_'.$method))
        { print 'WARNING '.$outputpath.'output2/'.$data[$d].'/'.'bg_'.$bgOrder[$b].'/prior_'.$prior.'/method_'.$method." : dir not found - SKIP\n"; next;}

        #now find the requested -i matrixfile for MotifRanking
        for(my $r=1; $r <= $repeats; $r++)
        {
          $matrixfile = $outputpath.'output2/'.$data[$d].'/'.'bg_'.$bgOrder[$b].'/prior_'.$prior.'/method_'.$method.'/'.'w'.$width.'_n'.$nbrMotifs.'_s'.$strand.'_x'.$overlap.'_M'.$maxInst.'_r'.$r.'.mtrx';
          if (!(-e $matrixfile))
          { print 'WARNING '.$matrixfile." : -i matrixfile not found - SKIP\n"; next;}

          #create appropriate outputfiles for MotifRanking
          $outputfile = $matrixfile; $outputfile =~ s/\.mtrx//;
          $textfile = $matrixfile; $textfile =~ s/\.mtrx//;
          my $treshpr = $tresh; $treshpr =~ s/\.//;
          $outputfile .= '_MR_s'.$shift.'_x'.$ovlap.'_r'.$rank.'_m'.$mode.'_t'.$treshpr.'.mtrx';
          $textfile .= '_MR_s'.$shift.'_x'.$ovlap.'_r'.$rank.'_m'.$mode.'_t'.$treshpr.'.txt';

          # and run the algorithm 
          $command = $driverpath.'applications/MotifRanking -i '.$matrixfile.' -o '.$outputfile.' -O '.$textfile.' -s '.$shift.' -x '.$ovlap.' -r '.$rank.' -m '.$mode.' -t '.$tresh;
          if ($mode == 2){$command .= ' -b '.$bgfile;}
          system($command);
          # report progress 
          if ($b==0 && $p==0 && $m==0 && $r==1) {print '#command : '.$command."\n";}
          else { print '#rerun with bg_'.$bgOrder[$b].'/prior_'.$prior.'/method_'.$method.'/run_'.$r."\n";} 
        }
      }
    }
  }
  print 'INFO @data[$d] = '.$data[$d]." - FINISHED\n";  
}
print 'INFO script finishes succesfully'."\n";         
           
######################################## SUBROUTINES
######################################################################################
#loading variables from file into main
######################################################################################
sub readsettings 
{ 
  my ($file) = @_;
  open(FI, $file);
  while(<FI>) 
  {
    my $line = $_; 
    chomp $line;
    if ($line =~ m/^\$/ || $line =~ m/^\@/){ eval($line);}
  }
  close(FI);
}

