#!/usr/bin/perl -w
#use strict;

##for local/server use, change path 
#my $driverpath = '/home/marleen/';
my $driverpath = '/users/sista/mclaeys/INCLUSive_for_users/';
## inclusive version 
$driverpath .= 'INCLUSive-3.1.5/';

## Date of last changes : 17 jan 2010

# script to run MS-3.1.5
# output goes to subdirectory /users/sista/mclaeys/INCLUSive_tests/MS-250_50_new

# perl command for this script :
#################################################### 
# perl runMS-250_50_new.pl $ARGV[0] $ARGV[1]...$ARGV[n]
####################################################
# $ARGV[i] = file with settings for different run-parameters 

####################################################
# subroutines
#################################################### 
# &readsettings 
####################################################

# fasta-path
my $fastapath;
my $outputpath;
if ( $driverpath =~ m/sista/) 
{ $fastapath = '/users/sista/vstorms/Datasets/Valerie/250_50_new/';
  $outputpath = '/users/sista/mclaeys/INCLUSive_tests/MS-250_50_new/';
}
else  
{ $fastapath = '/home/marleen/MS-250_50_new/RegulonDB/';
  $outputpath = $fastapath;
}

# variables to be loaded from the settingsfile
my @data; #= ('LexA');
#my @space; # = ('coregulation/');
my @bgOrder; # = (0,1,2,3);
my $bmatrix; # = 1
my $btrack; # = 1;
my @nbrInstPrior; # = ('0.25_0.25','0.9_0.25', 'u', 'e0_1', 'e0_0_1', 'e0_0.45_0.3_0.25','f0_1');
my @search; # = (1); 
my $runs; # = 100;
my $repeats; #10

# removed/changed from settingsfile : 
my $width; #to be read from the specific inputfile wM.txt
my $nbrMotifs = 1; 
my $overlap = '-'; # not applicable 
#my @tfbscoord; #to be read from the specific inputfile
my $strand = 0;
my $maxInst; #to be read from the specific inputfile

# read the ARGV 
my $nbrfiles = $#ARGV;
my $settingsfile;
my $argnum;

# local variables
my $command;
my $fastafile; my $fastafile1;
my $wMfile;
my $outpudir;
my $nbrBgOrders;
my $bgfile;
my $prior;
my $method;
my $outputdirsub;
my @split;


# repeat for each file
foreach $argnum (0 .. $nbrfiles)
{
  # load the file
  $settingsfile = $ARGV[$argnum];
  if (!(-e $settingsfile))
  { print 'ERROR '.$settingsfile." : file not found - SKIP\n"; next;}
  print 'Processing '.$settingsfile."...\n"; 

  # update the parameters as read from the settingsfile
  &readsettings($settingsfile); 

  # repeat for each dataset(=regulator)
  for(my $d = 0; $d < scalar(@data); $d++)
  {
    # get the fastafile with sequences
    $fastafile = $fastapath.$data[$d].'.fna';
    if (!(-e $fastafile))
    { print 'WARNING '.$fastafile." : fastafile not found - SKIP\n"; next;}
    print 'Processing '.$fastafile."...\n"; 
    ## copy the file so that the headerline only contains seq-id information
    $fastafile1 = $outputpath.$data[$d].'.fnacopy';
    open(FREAD, "<$fastafile");
    open(FWRITE, ">$fastafile1");
    while(<FREAD>)
    {
      my $line = $_;
      if ($line =~ m/^\>/) # > |89_GCTGCGCTTATCGACAGTTA|180_CCTGGCTTTCAGGGCAGCGT/20/phr
      { chomp($line);
        @split = split(/\//,$line);
        print FWRITE '>'.$split[2]."\n";
      }
      else { print FWRITE $line;}
    }
    close(FREAD); close(FWRITE);

    # get the motifwidth and maxInst (the wM.txt file should be available)
    $wMfile = $outputpath.'RegulonDB/'.$data[$d].'_wM.txt';
    if (!(-e $wMfile))
    { print 'WARNING '.$wMfile." : wMfile not found - SKIP MotifSampler\n"; next;}
    &readsettings($wMfile);
    if ( $width == 0 || $maxInst == 0)
    { print 'WARNING '.$wMfile.' : wrong $width/$maxInst format - SKIP MotifSampler'."\n"; next;} 

    # create regulator subdirectory for output
    $outputdir = $outputpath.'output/'.$data[$d].'/';
    if (!(-e $outputdir))
    { $command = 'mkdir -p '.$outputdir; system($command);}
  
    # repeat for each backgroundfile
    $nbrBgOrders = scalar(@bgOrder);
    for(my $b = 0; $b < $nbrBgOrders; $b++)
    {
      #find the backgroundfile (these should be available)
      $bgfile = $outputpath.'RegulonDB/'.$data[$d].'_'.$bgOrder[$b].'.bg';
      if (!(-e $bgfile))
      { print 'WARNING '.$bgfile." : backgroundfile not found - SKIP\n"; next;} 

      #repeat for each nbrInstprior
      for(my $p = 0; $p < scalar(@nbrInstPrior); $p++)
      {
        $prior = $nbrInstPrior[$p];
        #repeat for each search method
        for(my $m = 0; $m < scalar(@search); $m++)
        {
          $method = $search[$m];
          # create a meaningfull output-subdirectory
          $outputdirsub = $outputdir.'bg_'.$bgOrder[$b].'/prior_'.$prior.'/method_'.$method.'/';
          if (!(-e $outputdirsub))
          { $command = 'mkdir -p '.$outputdirsub; system($command); }

          # and run the algorithm for $repeats times
          for(my $r=1; $r <= $repeats; $r++)
          {
            $outputfile = $outputdirsub.'w'.$width.'_n'.$nbrMotifs.'_s'.$strand.'_x'.$overlap.'_M'.$maxInst.'_r'.$r.'.txt';
            $matrixfile = $outputdirsub.'w'.$width.'_n'.$nbrMotifs.'_s'.$strand.'_x'.$overlap.'_M'.$maxInst.'_r'.$r.'.mtrx';
            $trackfile = $outputdirsub.'w'.$width.'_n'.$nbrMotifs.'_s'.$strand.'_x'.$overlap.'_M'.$maxInst.'_r'.$r.'.track'; 
            $command = $driverpath.'applications/MotifSampler -f '.$fastafile1.
                   ' -b '.$bgfile.' -o '.$outputfile.' -r '.$runs.' -w '.$width.' -n '.$nbrMotifs.' -s '.$strand.' -M '.$maxInst.
                   ' -p '.$prior.' -S '.$method;
            if ($bmatrix) { $command .= ' -m '.$matrixfile; }
            if ($btrack) { $command .= ' -t '.$trackfile;}
            system($command);
            # report progress 
            if ($b==0 && $p==0 && $m==0 && $r==1) {print '#command : '.$command."\n";}
            else { print '#rerun with bg_'.$bgOrder[$b].'/prior_'.$prior.'/method_'.$method.'/run_'.$r."\n";}     
          }
        }
      }
    }
    #remove the $fastafile-copy
    $command = 'rm -f '.$fastafile1; system($command);
    print 'INFO @data[$d] = '.$data[$d]." - FINISHED\n";
  }   
}
print 'INFO script finishes succesfully'."\n";

######################################## SUBROUTINES
######################################################################################
#loading variables from file into main
######################################################################################
sub readsettings 
{ 
  my ($file) = @_;
  open(FI, $file);
  while(<FI>) 
  {
    my $line = $_; 
    chomp $line;
    if ($line =~ m/^\$/ || $line =~ m/^\@/){ eval($line);}
  }
  close(FI);
}


