#!/usr/bin/perl -w
use strict;
use DBI;

# script to compute the resume indicators for MS/MR/FC performance
# input is the program results from motif-detection without or with postprocessing 
# output is stored in dbi 

## Date of last changes : 17 juni 2010

###################line 749 : remove TFBs file : reset after testfaze

## !! MotifComparison BLiC : run with treshold 0.001 instead of 0.01 ! 
## !! rev5:FuzzyClustering : run with -p 0.10 instead of 0.05 and -m 0.8 instead of -1/0.5.
## !! rev6:Run only the parameterset of interest for each TF 
## !! rev6b:FuzzyClustering : run with -p 0.10 instead of 0.05 and -m 0.5 (ln249)
## !! rev6b:FuzzyClustering : run with -p 0.3 instead of 0.05 and -m 0.5 and i1(ln249)

# perl command for this script :
#################################################### 
# perl runResume-250_50_new_rev6b.pl $ARGV[0] $ARGV[1] 
####################################################
# $ARGV[0] = pseudo-count to be used in MotifComparison e.g 'scaled' for MS/MR or 0.001 for FC
# $ARGV[1] = settingsfile that was used for the motif-detection algorithm

## mark that this script is designed for one motif
## and for fixed motief length !
## the dbi tables should be created manually beforehand. 


##for local/server use, change path 
#my $driverpath = '/home/marleen/';
#my $dbi = 0;
my $driverpath = '/users/sista/mclaeys/INCLUSive_for_users/';
my $dbi = 1;
## inclusive version (for MotifComparison)
$driverpath .= 'INCLUSive-3.1.5/';
my $command;

# path
my $outputpath;
if ( $driverpath =~ m/sista/) { $outputpath = '/users/sista/mclaeys/INCLUSive_tests/MS-250_50_new/';}
else  { $outputpath = '/home/marleen/MS-250_50_new/';}

# variables to be loaded from the MS-settingsfile
my @data; #= ('LexA');
#my @space; # = ('coregulation/');
my @bgOrder; # = (0,1,2,3, Ecoli_2);
my $bmatrix; # = 1
my $btrack; # = 1;
my @nbrInstPrior; # = ('0.25_0.25','0.9_0.25', 'u', 'e0_1', 'e0_0_1', 'e0_0.45_0.3_0.25','f0_1');
my @search; # = (0,1); 
my $runs; # = 100;
my $repeats; #10
# removed/changed from settingsfile : 
my $width = 0; #to be read from the specific inputfile wM.txt
my $nbrMotifs = 1; 
my $overlap ='-'; # not applicable 
my $strand = 0;
my $maxInst = 0; #to be read from the specific inputfile
# for MR :
my @parMRrank = (); # = (1); #-r : for all data !! it will be only one value
#my @parMRshift = (); # not determing
#my @parMRovl = (); #determing : compute the value based on $width
my @parMRmode = ();
my @parMRtresh = (); 
# for FC :
my @parFCmode = ('basic'); # = ('basic', 'sym', 'asym'); #assign default it there would be no input

# read the ARGV 
my $nbrargv = $#ARGV;
if ( $nbrargv != 1 ) { print 'ERROR : two argv requested for perl script - end'."\n"; exit(-1);} 

my $ps = $ARGV[0];
my $scaledPs;
my $bScaled = 0;
if ($ps =~ m/scaled/){ $bScaled = 1;} # $ps to be read from wM.txt file as $scaledPs

my $settingsfile = $ARGV[1];
if (!(-e $settingsfile)) { print 'ERROR '.$settingsfile." : file not found - end\n"; exit(-1);} 


###########################################################################################
my $database = "motiefdetectie";
my $hostname = "dbserv.esat.kuleuven.be";
my $port = "3306";
my $username = "motiefdetectie";
my $password = 'gHte!ens';
my $dsn = "DBI:mysql:database=$database;host=$hostname;port=$port";
my $dbh;
if ($dbi){ $dbh = DBI->connect($dsn, $username, $password) or die("Could not connect to DBI!");}
#my $dbh = DBI->connect("DBI:mysql:dbserv.esat.kuleuven.be:motiefdetectie", 'motiefdetectie', 'ste5OLic');
my $table = 'rev6_perf_INCLUSivetestsMS25050new';
my $resumetable = 'rev6_resume_INCLUSivetestsMS25050new';
my $now;

#score float(50)
############################################################################################
#create table rev6_perf_INCLUSivetestsMS25050new (DateTime DATETIME, TF VARCHAR(5), TFBSmax int(1), bgm VARCHAR(8), prior VARCHAR(20), search INT(1), MSpar VARCHAR(150), MSaT_D1n int(4), MStT_F decimal(4,3), MStT_pvv decimal(4,3), MStT_sens decimal(4,3), MStT_CS decimal(4,3), MStT_LL int(4), MStT_KL decimal(4,3), MStT_BLiC decimal(17,14), MStT_id VARCHAR(150),MCKLpar VARCHAR(75), MStKL_F decimal(4,3), MStKL_pvv decimal(4,3), MStKL_sens decimal(4,3), MStKL_CS decimal(4,3), MStKL_LL int(4), MStKL_KL decimal(4,3), MStKL_BLiC decimal(17,14), MStKL_id VARCHAR(150), MCBLiCpar VARCHAR(75), MStBLiC_F decimal(4,3), MStBLiC_pvv decimal(4,3), MStBLiC_sens decimal(4,3), MStBLiC_CS decimal(4,3), MStBLiC_LL int(4), MStBLiC_KL decimal(4,3), MStBLiC_BLiC decimal(17,14), MStBLiC_id VARCHAR(150),MSaT_D11 int(4), MSaT_F decimal(4,3), MSaT_pvv decimal(4,3), MSaT_sens decimal(4,3), MSaT_KL decimal(4,3), MSaT_BLiC decimal(17,14), MSaKL_D11 int(4), MSaKL_F decimal(4,3), MSaKL_pvv decimal(4,3), MSaKL_sens decimal(4,3), MSaKL_KL decimal(4,3), MSaKL_BLiC decimal(17,14), MSaBLiC_D11 int(4), MSaBLiC_F decimal(4,3), MSaBLiC_pvv decimal(4,3), MSaBLiC_sens decimal(4,3), MSaBLiC_KL decimal(4,3), MSaBLiC_BLiC decimal(17,14), MRpar VARCHAR(150), MRaT_D1n int(4), MRaT_D11 int(4), MRtT_F decimal(4,3), MRtT_pvv decimal(4,3), MRtT_sens decimal(4,3), MRtT_sim int(3), MRtT_LL int(4), MRtT_CS decimal(4,3), MRtT_rank int(1), MRtT_KL decimal(4,3), MRtT_BLiC decimal(17,14),MRtT_id VARCHAR(150), MRtLL_F decimal(4,3), MRtLL_pvv decimal(4,3), MRtLL_sens decimal(4,3), MRtLL_sim int(3), MRtLL_LL int(4), MRtLL_CS decimal(4,3), MRtLL_rank int(1), MRtLL_KL decimal(4,3), MRtLL_BLiC decimal(17,14),MRtLL_id VARCHAR(150),MRtCS_F decimal(4,3), MRtCS_pvv decimal(4,3), MRtCS_sens decimal(4,3), MRtCS_sim int(3), MRtCS_LL int(4), MRtCS_CS decimal(4,3), MRtCS_rank int(1), MRtCS_KL decimal(4,3), MRtCS_BLiC decimal(17,14),MRtCS_id VARCHAR(150),MRtsim_F decimal(4,3), MRtsim_pvv decimal(4,3), MRtsim_sens decimal(4,3), MRtsim_sim int(3), MRtsim_LL int(4), MRtsim_CS decimal(4,3), MRtsim_rank int(1), MRtsim_KL decimal(4,3), MRtsim_BLiC decimal(17,14), MRtsim_id VARCHAR(150),FCpar VARCHAR(75), FCaT_D1n int(4), FCaT_D11 int(4), FCtT_F decimal(4,3), FCtT_pvv decimal(4,3), FCtT_sens decimal(4,3), FCtT_aocc decimal(4,2), FCtT_aISS1 decimal(4,3), FCtT_aISS0 decimal(4,3), FCtT_CS decimal(4,3), FCtT_rank int(1), FCtT_KL decimal(4,3), FCtT_BLiC decimal(17,14), FCtT_id VARCHAR(150), FCt1_F decimal(4,3), FCt1_pvv decimal(4,3), FCt1_sens decimal(4,3), FCt1_aocc decimal(4,2), FCt1_aISS1 decimal(4,3), FCt1_aISS0 decimal(4,3), FCt1_CS decimal(4,3), FCt1_rank int(1), FCt1_KL decimal(4,3), FCt1_BLiC decimal(17,14),FCt1_id VARCHAR(150), FCtCS_F decimal(4,3), FCtCS_pvv decimal(4,3), FCtCS_sens decimal(4,3), FCtCS_aocc decimal(4,2), FCtCS_aISS1 decimal(4,3), FCtCS_aISS0 decimal(4,3), FCtCS_CS decimal(4,3), FCtCS_rank int(1), FCtCS_KL decimal(4,3), FCtCS_BLiC decimal(17,14), FCtCS_id VARCHAR(150), FCtaocc_F decimal(4,3), FCtaocc_pvv decimal(4,3), FCtaocc_sens decimal(4,3), FCtaocc_aocc decimal(4,2), FCtaocc_aISS1 decimal(4,3), FCtaocc_aISS0 decimal(4,3), FCtaocc_CS decimal(4,3), FCtaocc_rank int(1), FCtaocc_KL decimal(4,3), FCtaocc_BLiC decimal(17,14),FCtaocc_id VARCHAR(150), primary key (DateTime,TF,bgm,prior,search,MSpar, MRpar, FCpar, MCKLpar, MCBLiCpar));
############################################################################################

#create table rev6_resume_INCLUSivetestsMS25050new (DateTime DATETIME, TF VARCHAR(5), TFBSmax int(1), bgm VARCHAR(8), prior VARCHAR(20), search INT(1), MSpar VARCHAR(150), MStT_repeat int(2), MSaT_D1n decimal(4,1), MStT_F decimal(4,3), MStT_pvv decimal(4,3), MStT_sens decimal(4,3), MStT_CS decimal(4,3), MStT_LL int(4), MStT_KL decimal(4,3), MStT_BLiC decimal(17,14), MCKLpar VARCHAR(75), MStKL_F decimal(4,3), MStKL_pvv decimal(4,3), MStKL_sens decimal(4,3), MStKL_CS decimal(4,3), MStKL_LL int(4), MStKL_KL decimal(4,3), MStKL_BLiC decimal(17,14), MCBLiCpar VARCHAR(75), MStBLiC_F decimal(4,3), MStBLiC_pvv decimal(4,3), MStBLiC_sens decimal(4,3), MStBLiC_CS decimal(4,3), MStBLiC_LL int(4), MStBLiC_KL decimal(4,3), MStBLiC_BLiC decimal(17,14), MSaT_D11 decimal(4,1), MSaT_F decimal(4,3), MSaT_pvv decimal(4,3), MSaT_sens decimal(4,3), MSaT_KL decimal(4,3), MSaT_BLiC decimal(17,14), MSaKL_D11 decimal(4,1), MSaKL_F decimal(4,3), MSaKL_pvv decimal(4,3), MSaKL_sens decimal(4,3), MSaKL_KL decimal(4,3), MSaKL_BLiC decimal(17,14), MSaBLiC_D11 decimal(4,1), MSaBLiC_F decimal(4,3), MSaBLiC_pvv decimal(4,3), MSaBLiC_sens decimal(4,3), MSaBLiC_KL decimal(4,3), MSaBLiC_BLiC decimal(17,14), MRpar VARCHAR(150), MRtT_repeat int(2), MRaT_D1n decimal(4,1), MRaT_D11 decimal(4,1), MRtT_F decimal(4,3), MRtT_pvv decimal(4,3), MRtT_sens decimal(4,3), MRtT_sim int(3), MRtT_LL int(4), MRtT_CS decimal(4,3), MRtT_rank decimal(2,1), MRtT_KL decimal(4,3), MRtT_BLiC decimal(17,14), MRtLL_F decimal(4,3), MRtLL_pvv decimal(4,3), MRtLL_sens decimal(4,3), MRtLL_sim int(3), MRtLL_LL int(4), MRtLL_CS decimal(4,3), MRtLL_rank decimal(2,1), MRtLL_KL decimal(4,3), MRtLL_BLiC decimal(17,14), MRtCS_F decimal(4,3), MRtCS_pvv decimal(4,3), MRtCS_sens decimal(4,3), MRtCS_sim int(3), MRtCS_LL int(4), MRtCS_CS decimal(4,3), MRtCS_rank decimal(2,1), MRtCS_KL decimal(4,3), MRtCS_BLiC decimal(17,14), MRtsim_F decimal(4,3), MRtsim_pvv decimal(4,3), MRtsim_sens decimal(4,3), MRtsim_sim int(3), MRtsim_LL int(4), MRtsim_CS decimal(4,3), MRtsim_rank decimal(2,1), MRtsim_KL decimal(4,3), MRtsim_BLiC decimal(17,14),  FCpar VARCHAR(75), FCtT_repeat int(2), FCaT_D1n decimal(4,1), FCaT_D11 decimal(4,1), FCtT_F decimal(4,3), FCtT_pvv decimal(4,3), FCtT_sens decimal(4,3), FCtT_aocc decimal(4,2), FCtT_aISS1 decimal(4,3), FCtT_aISS0 decimal(4,3), FCtT_CS decimal(4,3), FCtT_rank decimal(2,1), FCtT_KL decimal(4,3), FCtT_BLiC decimal(17,14),  FCt1_F decimal(4,3), FCt1_pvv decimal(4,3), FCt1_sens decimal(4,3), FCt1_aocc decimal(4,2), FCt1_aISS1 decimal(4,3), FCt1_aISS0 decimal(4,3), FCt1_CS decimal(4,3), FCt1_rank decimal(2,1), FCt1_KL decimal(4,3), FCt1_BLiC decimal(17,14), FCtCS_F decimal(4,3), FCtCS_pvv decimal(4,3), FCtCS_sens decimal(4,3), FCtCS_aocc decimal(4,2), FCtCS_aISS1 decimal(4,3), FCtCS_aISS0 decimal(4,3), FCtCS_CS decimal(4,3), FCtCS_rank decimal(2,1), FCtCS_KL decimal(4,3), FCtCS_BLiC decimal(17,14), FCtaocc_F decimal(4,3), FCtaocc_pvv decimal(4,3), FCtaocc_sens decimal(4,3), FCtaocc_aocc decimal(4,2), FCtaocc_aISS1 decimal(4,3), FCtaocc_aISS0 decimal(4,3), FCtaocc_CS decimal(4,3), FCtaocc_rank decimal(2,1), FCtaocc_KL decimal(4,3), FCtaocc_BLiC decimal(17,14), primary key (DateTime,TF,bgm,prior,search,MSpar, MRpar, FCpar, MCKLpar, MCBLiCpar));
#############################################################################################

# read the settings to reconstruct format of the MS/MR/FC-inputfiles
&readsettings($settingsfile);


if (scalar(@data) != scalar(@nbrInstPrior) || scalar(@data) != scalar(@search) || scalar(@data) != scalar(@bgOrder))
{print 'ERROR '.$settingsfile." : scalars = ".scalar(@data).'/'.scalar(@bgOrder).'/'.scalar(@nbrInstPrior).'/'.scalar(@search)." - individual listing of prior/search per TF ! - exit\n"; exit(-1);}

# for MR :
my $MRrank= $parMRrank[0];
my $shift = 0; # to be set as $width !!
my $ovlap = 0; # to be set as half of $width !!
my $tresh = 0.4; # the general default 
if (scalar(@parMRtresh) == 1){ $tresh = $parMRtresh[0];}
my $treshpr = $tresh; $treshpr =~ s/\.//;
my $mode = 0; 
if (scalar(@parMRmode) == 1){ $mode = $parMRmode[0];}
# for FC :
########move down#########if (!($posttype =~ m/clust/)){ @parFCmode = (''); }

my $posttype; 
my $MSkolom;
my $MRkolom = '';
my $FCkolom = '';
my $MCKLkolom = '';
my $MCBLiCkolom = '';
my $resMSkolom = '';

# variables to put into table
my $D1n; my $D11; my$F; my$pvv; my$sens; my$CS;my$LL;my$KL;my$BLiC; my $sim; my$rank; my$aocc; my$aISS1; my$aISS0;

                       
#to be loaded from wMfile to put in DBI-resume as TF info
# TFcs double(6,3), TFBSw int(2), TFBSnbr int(3), TFBSmax int(1), Seqnbr int(3)
my $pwmCS; 
# my $width; already listed above
my $totalInst;
# my $maxInst; already listed above
my $nbrSeq; 

#local variables
my $bOverlap;
my $inputdir;
my $bgfile;
my $wMfile;
my $annotatie;
my $bgorder;
my $bgm;
my $prior; 
my $method;
my $datai;
my $txtfile;
my $mtrxfile; 
my $outputfile;
my $resoutputfile;
my $annotpwm;
my @array;
my @post = ('none','rank','clust');
my $Q; my $QQ; my $bNone; my $bRank; my $bClust;
my $kl; my $blic; my $distance = 0; 
my @resume;
for( my $q=0; $q < 50; $q++) { push(@resume, 0);}

# repeat for each TF
for(my $d = 0; $d < scalar(@data); $d++)
{
  $datai = $data[$d];
  $width = 0;$maxInst = 0;$pwmCS = 0;$totalInst = 0; $nbrSeq = 0; #reset

  # get the information from wM.txt file 
  $wMfile = $outputpath.'RegulonDB/'.$datai.'_wM.txt';
  if (!(-e $wMfile))
  { print 'WARNING '.$wMfile." : wMfile not found - SKIP runPerf-250_50_new\n"; next;}
  &readsettings($wMfile);
  if ($bScaled) { $ps = $scaledPs;}
  my $MCkolomm = 'MC_'.$ps;

  # check if all parameters are read correctly
  if ( $width == 0 || $maxInst == 0 || $pwmCS == 0 || $totalInst == 0 || $nbrSeq == 0 || $ps == 0)
  { print 'WARNING '.$wMfile.' : some parameters equal zero - SKIP'."\n"; next;} 
  #set MC-overlap as half of $width (just as we did for MR)
  $ovlap = int($width/2);
  $shift = $width;

  # $inputdir where textfiles are available
  $inputdir = $outputpath.'output/'.$datai.'/';
  if (!(-e $inputdir))
  { print 'WARNING '.$inputdir." : directory not found - SKIP\n"; next;}

  # create a hash of arrays with results from annotatie-file for this TF
  $annotatie =  $outputpath.'RegulonDB/'.$datai.'_annotatie.txt';
  if (!(-e $annotatie))
  { print 'WARNING '.$annotatie." : annotatie file not found - SKIP further performance computation !\n"; next;}
  my ($annotP, $annothashref) = &annotatedhash($annotatie);
  if ( $annotP < 1 ) 
  { print 'WARNING '.$annotatie." : no annotated TFBS information available - SKIP further performance !\n"; next;}
  ## do a little check :
  if ($annotP != $totalInst)
  { print 'WARNING $annotP/'.$annotP.' != $totalInst/'.$totalInst." - CHECK THIS !!!!\n"; }

  # repeat for each backgroundfile
  for(my $b = 0; $b < 1; $b++) #rev6 : only one time
  {
    $bgorder = $bgOrder[$d]; # specific for each TF
    if ( $bgorder =~ m/Ecoli/){ $bgfile = $outputpath.'RegulonDB/'.$bgorder.'.bg'; $bgm = $bgorder;}
    else { $bgfile = $outputpath.'RegulonDB/'.$datai.'_'.$bgorder.'.bg'; $bgm = 'data_'.$bgorder;}

    #repeat for each nbrInstprior
    for(my $p = 0; $p < 1; $p++)#rev6 : only one time
    {
      $prior = $nbrInstPrior[$d];# specific for each TF
      #repeat for each search method
      for(my $m = 0; $m < 1; $m++)#rev6 : only one time
      {
        $method = $search[$d];# specific for each TF
        #check on parameters
        if (!(-e $inputdir.'bg_'.$bgorder.'/prior_'.$prior.'/method_'.$method))
        { print 'WARNING '.$inputdir.'bg_'.$bgorder.'/prior_'.$prior.'/method_'.$method." : directory not found - SKIP\n"; next;}

        ## set for resume computation
        $resMSkolom = 'w'.$width.'_n'.$nbrMotifs.'_s'.$strand.'_x'.$overlap.'_M'.$maxInst;
        $now = &time;

        #repeat for each type of postprocessing :

        for(my $po =0; $po < 3; $po++)
        { 
          $posttype = $post[$po];
          # lines moved from higher up :
          #if (!($posttype =~ m/clust/)){ @parFCmode = (''); }
          #else { 
          @parFCmode = ('basic');
          #} ## assumed for now with RegulonDB tests 
          if ($posttype =~ m/none/)
          { $Q = 41; $QQ= 7; $bNone = 1; $bRank = 0; $bClust = 0;}
          if ($posttype =~ m/rank/) 
          { $Q = 39; $QQ= 9; $bNone = 0; $bRank = 1; $bClust = 0;}
          if ($posttype =~ m/clust/) 
          { $Q = 43; $QQ= 10; $bNone = 0; $bRank = 0; $bClust = 1;}

          # resume array with at maximum $Q entries, all set to zero (just all after one another)  
          for( my $q=0; $q < 50; $q++) { $resume[$q] = 0;}

          #reset :
          $MSkolom = '';
          $MRkolom = '_MR_s'.$shift.'_x'.$ovlap.'_r'.$MRrank.'_m'.$mode.'_t'.$treshpr;
          $FCkolom = '_FC_t'.$parFCmode[0].'_p03_m05i1'; ###################!!!!!
          $MCKLkolom = 'KL_0.4';##fixed for this script, change here ###and in subroutine !!!#### if you want to rerun 
          $MCBLiCkolom = 'BLiC_0.001'; ##fixed for this script, change here ###and in subroutine !!!#### if you want to rerun 



          #repeat
          for(my $r=1; $r <= $repeats; $r++)
          {
            $MSkolom = 'w'.$width.'_n'.$nbrMotifs.'_s'.$strand.'_x'.$overlap.'_M'.$maxInst.'_r'.$r;
          
            # find the requested file with results from MotifSampling|MotifRanking|FuzzyClustering     
            $mtrxfile = $inputdir.'bg_'.$bgorder.'/prior_'.$prior.'/method_'.$method.'/'.$MSkolom;
            $txtfile = $inputdir.'bg_'.$bgorder.'/prior_'.$prior.'/method_'.$method.'/'.$MSkolom;
            #create appropriate outputfile for performance results
            $outputfile = $inputdir.'bg_'.$bgorder.'/prior_'.$prior.'/method_'.$method.'/'.$MSkolom;
            if ($bRank) { $mtrxfile.= $MRkolom; $txtfile.= $MRkolom;$outputfile .= $MRkolom; }
            elsif ($bClust) { $mtrxfile.= $FCkolom;$txtfile.= $FCkolom;$outputfile .= $FCkolom;}
            $mtrxfile.= '.mtrx';
            $txtfile.= '.txt';
            $outputfile .= $MCkolomm.'.perf';

#print "textfile = $txtfile ############# matrixfile = $mtrxfile\n";

            ##open(OUTPUT, ">$outputfile");
            #results will also be printed to dbi table
            ##print OUTPUT '#performance for '.$outputfile."\n";

            #create appropriate outputfile for resume results and reset entries
            if ($r == 1)
            { $resoutputfile = $inputdir.'bg_'.$bgorder.'/prior_'.$prior.'/method_'.$method.'/'.$resMSkolom.$MCkolomm.'.resume';
              if ($bRank) { $resoutputfile.= $MRkolom; }
              elsif ($bClust) { $resoutputfile.= $FCkolom;}
              $resoutputfile .= $MCkolomm.'.resume';
            }

            
            if (!(-e $mtrxfile))
            { print 'WARNING '.$mtrxfile." : requested mtrx-file not found - SKIP\n"; $D1n = -1; } #no valid output $D1 stays -1
            else
            { ## compute number of retrieved motifs (D1) -> count number of #ID = box lines in matrixfile, scaled with -n  
              $D1n = &countmatrices($mtrxfile); $D1n /= $nbrMotifs;    
            }

            # insert the set into performance and resume table (all other values will be updated later)
            if ($bNone)
            { $command = "insert into $table (DateTime, TF, TFBSmax, bgm, prior, search, MSpar, MRpar, FCpar, MCKLpar, MCBLiCpar, MSaT_D1n) values ('$now','$datai','$maxInst' ,'$bgm', '$prior', $method, '$MSkolom', '$MRkolom', '$FCkolom', '$MCKLkolom', '$MCBLiCkolom', $D1n)";
            }
            elsif ($bRank)
            { $command = "update $table set MRaT_D1n = $D1n where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
            }
            else #bClust
            { $command = "update $table set FCaT_D1n = $D1n where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
            }
            $dbh->do("$command");

            if ($D1n <= 0) #no valid file (-1) or no output in file (0)
            {   
              # do nothing for @resume
            }
            else #only do this performance part if $D1 > 0 
            {

            # add D1n result to resume-repeat and resume-D1n
            $resume[0] += 1; $resume[1] += $D1n; 

            #### retrieve the number of correct matrices out of D1
            $annotpwm =  $outputpath.'RegulonDB/'.$datai.'_pwm_ps'.$ps.'.mtrx'; ############# 2010/maart/07
            if (!(-e $annotpwm))
            { print 'WARNING '.$annotpwm." : pwm file not found - SKIP further performance computation !\n"; next;}

            # use heuristic TFBS based approach to identify correct motifs(allTFBS) and the very best motif(topTFBS)
            my $txt2file; # file with TFBS info of D1-motifs (to be created for MR based on MS.txt file)
            if (!(-e $txtfile))
            { print 'WARNING '.$txtfile." : requested txt-file not found - SKIP\n"; }
            else
            {
              $txt2file = $txtfile;
              if ($bRank) # get required TFBS-info from the MS-txtfile - with the motif/TFBSs in ranked order !
              { $txt2file = &gettfbsinfo($txtfile);}

              # run MotifComparison KL
              @array = (0.4, $shift, $ovlap, 0); ## @KL = -t/-s/-x/-l
              my $KLoutfile = &motifcomparison($mtrxfile, $annotpwm, $ps, $posttype, @array);

              # run MotifComparison BLiC  
              my $BLiCoutfile; 
              if (-e $bgfile)
              { @array = (0.001, $shift, $ovlap, 1, $width, $bgfile ); ## @BLIC = -t/-s/-x/-l/-n(/-b)(/-p)           
                $BLiCoutfile = &motifcomparison($mtrxfile, $annotpwm, $ps, $posttype, @array); 
              } 
              else { print 'WARNING '.$bgfile." : bg file not found - SKIP BLIC-MotifComparison\n";}

              # compute/print/store all and best correct motifs (topTFBS and allTFBS-D11)
              my ($TFBSoutfile, $topTFBSid, $top1, $topCS, $topsim, $topaocc, @allTFBSids) = &TFBScomparison($txt2file,$width,$annotP,$annothashref,\@resume, $posttype, $txtfile);

#print 'TFBScomparison output = '."topTFBS = $topTFBSid - top1 = $top1 - topCS = $topCS - topsim = $topsim - topaocc = $topaocc\n";

              # for topTFBS : get the KL/BLiC score, add to resume/print to DBI
              if ($topTFBSid ne 'NULL')
              {
                ($kl) = &getMCscore($topTFBSid, $KLoutfile); ($blic) = &getMCscore($topTFBSid, $BLiCoutfile);
                if ($bNone)
                { $command = "update $table set MStT_KL = $kl , MStT_BLiC = $blic where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                  $resume[25] += $kl; $resume[26] += $blic;
                  $dbh->do("$command"); 
                  # do the same for @allTFBS
                  $kl = 0; $blic = 0; my $id;
                  for(my $i = 0; $i< scalar(@allTFBSids); $i++)
                  { $id = $allTFBSids[$i];
                    $kl += &getMCscore($id, $KLoutfile); $blic += &getMCscore($id, $BLiCoutfile);
                  }
                  if (scalar(@allTFBSids) > 0) {$kl /= scalar(@allTFBSids); $blic /= scalar(@allTFBSids); }
                  $command = "update $table set MSaT_KL = $kl , MSaT_BLiC = $blic where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                  $resume[6] += $kl; $resume[7] += $blic;
                }
                elsif ($bRank)
                { $command = "update $table set MRtT_KL = $kl , MRtT_BLiC = $blic where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                $resume[10] += $kl; $resume[11] += $blic; 
                }
                else
                { $command = "update $table set FCtT_KL = $kl , FCtT_BLiC = $blic where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                $resume[11] += $kl; $resume[12] += $blic; 
                }

                $dbh->do("$command");
              }#end of if topTFBS ne NULL

              # for top1 : get the KL/BLiC score, add to resume/print to DBI
              if (!$bNone && $top1 ne 'NULL')
              {
                ($kl) = &getMCscore($top1, $KLoutfile); ($blic) = &getMCscore($top1, $BLiCoutfile);
                if ($bRank)
                { $command = "update $table set MRtLL_KL = $kl , MRtLL_BLiC = $blic where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                  $resume[19] += $kl; $resume[20] += $blic; 
                }
                else
                { $command = "update $table set FCt1_KL = $kl , FCt1_BLiC = $blic where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                  $resume[21] += $kl; $resume[22] += $blic; 
                }
                $dbh->do("$command");
              }#end of if top1 ne NULL

              # for topCS : get the KL/BLiC score, add to resume/print to DBI
              if (!$bNone && $topCS ne 'NULL')
              {
                ($kl) = &getMCscore($topCS, $KLoutfile); ($blic) = &getMCscore($topCS, $BLiCoutfile);
                if ($bRank)
                { $command = "update $table set MRtCS_KL = $kl ,  MRtCS_BLiC = $blic where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                  $resume[28] += $kl; $resume[29] += $blic; 
                }
                else
                { $command = "update $table set FCtCS_KL = $kl  , FCtCS_BLiC = $blic where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                  $resume[31] += $kl; $resume[32] += $blic; 
                }
                $dbh->do("$command");
              }#end of if topCS ne NULL

              # for topsim : get the KL/BLiC score, add to resume/print to DBI
              if ($bRank && $topCS ne 'NULL')
              {
                ($kl) = &getMCscore($topsim, $KLoutfile); ($blic) = &getMCscore($topsim, $BLiCoutfile);
                $command = "update $table set MRtsim_KL = $kl , MRtsim_BLiC = $blic where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                 $resume[37] += $kl; $resume[38] += $blic; 
                $dbh->do("$command");
              }
              # for topaocc : get the KL/BLiC score, add to resume/print to DBI
              if ($bClust && $topaocc ne 'NULL')
              {
                ($kl) = &getMCscore($topaocc, $KLoutfile); ($blic) = &getMCscore($topaocc, $BLiCoutfile);
              $command = "update $table set FCtaocc_KL = $kl , FCtaocc_BLiC = $blic where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                $resume[41] += $kl; $resume[42] += $blic; 
                $dbh->do("$command");
              }#end of if topaocc ne NULL
   
              # for bNone : MotifComparison evaluation
              if ($bNone)
              {   
                &MCevaluation($KLoutfile, $BLiCoutfile, $TFBSoutfile, \@resume);
                &MCevaluation($BLiCoutfile, $KLoutfile, $TFBSoutfile, \@resume);
              }


              #remove the temporary files
              #if (-e $KLoutfile) { $command = 'rm '.$KLoutfile; system($command);}
              #if (-e $BLiCoutfile) { $command = 'rm '.$BLiCoutfile; system($command);}
              #if (-e $TFBSoutfile) { $command = 'rm '.$TFBSoutfile; system($command);}
              #if (-e $txt2file) { $command = 'rm '.$txt2file; system($command);}
            }

            } ## end of only do this performance part if $D1 > 0 
            
            ##close(OUTPUT);
            # report progress 
            if ($b==0 && $p==0 && $m==0 && $r==1) {print '#performance for : '.$outputfile."\n";}
            else { print '#repeat perf for '.$outputfile."-end\n";}

            # report the resumed output in  table
            if ($r == $repeats)
            {
              my $stat = $resume[0];
              # insert the set into resume table (all other values will be updated later)
              if ($bNone)
              { $command = "insert into $resumetable (DateTime, TF, TFBSmax, bgm, prior, search, MSpar, MRpar, FCpar, MCKLpar, MCBLiCpar, MStT_repeat) values ('$now','$datai','$maxInst' ,'$bgm', '$prior', $method, '$resMSkolom', '$MRkolom', '$FCkolom', '$MCKLkolom', '$MCBLiCkolom', $stat)";
              }
              elsif ($bRank)
              { $command = "update $resumetable set MRtT_repeat = $stat where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$resMSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
              }
              else #bClust
              { $command = "update $resumetable set FCtT_repeat = $stat where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$resMSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
              }
              $dbh->do("$command");

              if ($stat > 0) # else there are no repeats that gave any outputs
              {
                # rescale all resume computations
                for(my$q = 1; $q < $Q; $q++) { if ($stat > 0) {$resume[$q] /= $stat;}}

                if ($bNone)
                { 
                  $D1n = $resume[1]; $D11 = $resume[2]; $F = $resume[3]; $pvv = $resume[4]; $sens= $resume[5]; $KL = $resume[6]; $BLiC = $resume[7];       $command = "update $resumetable set MSaT_D1n = $D1n ,  MSaT_D11 = $D11 , MSaT_F = $F  ,  MSaT_pvv = $pvv  ,  MSaT_sens = $sens ,  MSaT_KL = $KL  ,  MSaT_BLiC = $BLiC where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$resMSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                  $dbh->do("$command"); 
                  $D11 = $resume[8];  $F = $resume[9]; $pvv = $resume[10]; $sens= $resume[11]; $KL = $resume[12]; $BLiC = $resume[13];
                  $command = "update $resumetable set MSaKL_D11 = $D11 ,  MSaKL_F = $F  ,  MSaKL_pvv = $pvv  ,  MSaKL_sens = $sens  ,  MSaKL_KL = $KL  ,  MSaKL_BLiC = $BLiC where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$resMSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                  $dbh->do("$command");
                  $D11 = $resume[14];  $F = $resume[15]; $pvv = $resume[16]; $sens= $resume[17]; $KL = $resume[18]; $BLiC = $resume[19];
                  $command = "update $resumetable set MSaBLiC_D11 = $D11  ,  MSaBLiC_F = $F  ,  MSaBLiC_pvv = $pvv ,  MSaBLiC_sens = $sens  ,  MSaBLiC_KL = $KL  , MSaBLiC_BLiC = $BLiC where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$resMSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                  $dbh->do("$command"); 
                  $F = $resume[20]; $pvv = $resume[21]; $sens= $resume[22]; $CS = $resume[23]; $LL = $resume[24];$KL = $resume[25]; $BLiC = $resume[26];
                  $command = "update $resumetable set MStT_F = $F  ,   MStT_pvv = $pvv   ,   MStT_sens = $sens   ,   MStT_CS = $CS   ,   MStT_LL = $LL  ,   MStT_KL = $KL   ,   MStT_BLiC = $BLiC where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$resMSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                  $dbh->do("$command"); 
                  $F = $resume[27]; $pvv = $resume[28]; $sens= $resume[29]; $CS = $resume[30]; $LL = $resume[31];$KL = $resume[32]; $BLiC = $resume[33];
                  $command = "update $resumetable set MStKL_F = $F ,  MStKL_pvv = $pvv  ,  MStKL_sens = $sens  , MStKL_CS = $CS  ,  MStKL_LL = $LL  ,  MStKL_KL = $KL  ,  MStKL_BLiC = $BLiC where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$resMSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                  $dbh->do("$command");
                  $F = $resume[34]; $pvv = $resume[35]; $sens= $resume[36]; $CS = $resume[37]; $LL = $resume[38];$KL = $resume[39]; $BLiC = $resume[40];
                  $command = "update $resumetable set MStBLiC_F = $F  ,  MStBLiC_pvv = $pvv , MStBLiC_sens = $sens  ,  MStBLiC_CS = $CS ,  MStBLiC_LL = $LL  ,  MStBLiC_KL = $KL  ,  MStBLiC_BLiC = $BLiC where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$resMSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                  $dbh->do("$command");  
                }
                elsif ($bRank)
                { $D1n = $resume[1]; $D11 = $resume[2]; 
                  $command = "update $resumetable set MRaT_D1n = $D1n  ,   MRaT_D11 = $D11 where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$resMSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                  $dbh->do("$command");
                  $F = $resume[3]; $pvv = $resume[4]; $sens= $resume[5]; $sim = $resume[6]; $CS = $resume[8]; $LL = $resume[7]; $rank = $resume[9]; $KL = $resume[10]; $BLiC = $resume[11];
                  $command = "update $resumetable set MRtT_F = $F  ,  MRtT_pvv = $pvv  ,  MRtT_sens = $sens  ,  MRtT_CS = $CS  ,  MRtT_LL = $LL  ,  MRtT_KL = $KL  ,  MRtT_BLiC = $BLiC  , MRtT_sim = $sim  ,  MRtT_rank = $rank where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$resMSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                  $dbh->do("$command");
                  $F = $resume[12]; $pvv = $resume[13]; $sens= $resume[14]; $sim = $resume[15]; $CS = $resume[17]; $LL = $resume[16]; $rank = $resume[18]; $KL = $resume[19]; $BLiC = $resume[20];
                  $command = "update $resumetable set MRtLL_F = $F ,  MRtLL_pvv = $pvv ,  MRtLL_sens = $sens  ,  MRtLL_CS = $CS  ,  MRtLL_LL = $LL  , MRtLL_KL = $KL  ,  MRtLL_BLiC = $BLiC  ,  MRtLL_sim = $sim  ,  MRtLL_rank = $rank where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$resMSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                  $dbh->do("$command"); 
                  $F = $resume[21]; $pvv = $resume[22]; $sens= $resume[23]; $sim = $resume[24]; $CS = $resume[26]; $LL = $resume[25]; $rank = $resume[27]; $KL = $resume[28]; $BLiC = $resume[29];
                  $command = "update $resumetable set MRtCS_F = $F  ,  MRtCS_pvv = $pvv  ,  MRtCS_sens = $sens  ,  MRtCS_CS = $CS  ,  MRtCS_LL = $LL  ,  MRtCS_KL = $KL  ,  MRtCS_BLiC = $BLiC  ,  MRtCS_sim = $sim  ,  MRtCS_rank = $rank where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$resMSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                  $dbh->do("$command");
                  $F = $resume[30]; $pvv = $resume[31]; $sens= $resume[32]; $sim = $resume[33]; $CS = $resume[35]; $LL = $resume[34]; $rank = $resume[36]; $KL = $resume[37]; $BLiC = $resume[38];
                  $command = "update $resumetable set MRtsim_F = $F  ,  MRtsim_pvv = $pvv  ,  MRtsim_sens = $sens , MRtsim_CS = $CS  ,  MRtsim_LL = $LL  ,  MRtsim_KL = $KL  ,  MRtsim_BLiC = $BLiC  ,  MRtsim_sim = $sim  , MRtsim_rank = $rank where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$resMSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                  $dbh->do("$command");
                }
                else #bClust
                { $D1n = $resume[1]; $D11 = $resume[2]; 
                  $command = "update $resumetable set FCaT_D1n = $D1n  ,   FCaT_D11 = $D11 where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$resMSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                  $dbh->do("$command");
                  $F = $resume[3]; $pvv = $resume[4]; $sens= $resume[5]; $aocc = $resume[6]; $aISS1 = $resume[7]; $aISS0 = $resume[8]; $CS = $resume[9]; $rank = $resume[10]; $KL = $resume[11]; $BLiC = $resume[12];
                  $command = "update $resumetable set FCtT_F = $F  , FCtT_pvv = $pvv  ,  FCtT_sens = $sens  ,  FCtT_CS = $CS  ,  FCtT_rank = $rank ,  FCtT_KL = $KL  ,  FCtT_BLiC = $BLiC  ,  FCtT_aocc = $aocc  , FCtT_aISS0 = $aISS0  ,  FCtT_aISS1 = $aISS1 where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$resMSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                  $dbh->do("$command");
                  $F = $resume[13]; $pvv = $resume[14]; $sens= $resume[15]; $aocc = $resume[16]; $aISS1 = $resume[17]; $aISS0 = $resume[18]; $CS = $resume[19]; $rank = $resume[20]; $KL = $resume[21]; $BLiC = $resume[22];
                  $command = "update $resumetable set FCt1_F = $F  ,  FCt1_pvv = $pvv  ,  FCt1_sens = $sens  , FCt1_CS = $CS  ,  FCt1_rank = $rank  ,  FCt1_KL = $KL  ,  FCt1_BLiC = $BLiC  ,  FCt1_aocc = $aocc  , FCt1_aISS0 = $aISS0  ,  FCt1_aISS1 = $aISS1 where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$resMSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                  $dbh->do("$command");
                  $F = $resume[23]; $pvv = $resume[24]; $sens= $resume[25]; $aocc = $resume[26]; $aISS1 = $resume[27]; $aISS0 = $resume[28]; $CS = $resume[29]; $rank = $resume[30]; $KL = $resume[31]; $BLiC = $resume[32];
                  $command = "update $resumetable set FCtCS_F = $F  ,  FCtCS_pvv = $pvv  ,  FCtCS_sens = $sens  ,  FCtCS_CS = $CS  ,  FCtCS_rank = $rank  ,  FCtCS_KL = $KL  ,  FCtCS_BLiC = $BLiC ,  FCtCS_aocc = $aocc  ,  FCtCS_aISS0 = $aISS0  ,  FCtCS_aISS1 = $aISS1 where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$resMSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                  $dbh->do("$command");
                  $F = $resume[33]; $pvv = $resume[34]; $sens= $resume[35]; $aocc = $resume[36]; $aISS1 = $resume[37]; $aISS0 = $resume[38]; $CS = $resume[39]; $rank = $resume[40]; $KL = $resume[41]; $BLiC = $resume[42];
                  $command = "update $resumetable set FCtaocc_F = $F ,  FCtaocc_pvv = $pvv ,  FCtaocc_sens = $sens ,  FCtaocc_CS = $CS  ,  FCtaocc_rank = $rank  ,  FCtaocc_KL = $KL  ,  FCtaocc_BLiC = $BLiC  , FCtaocc_aocc = $aocc  ,  FCtaocc_aISS0 = $aISS0  , FCtaocc_aISS1 = $aISS1 where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$resMSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
                  $dbh->do("$command");
                }
              }
              # report progress 
              if ($b==0 && $p==0 ) {print '#resume for : '.$resoutputfile."\n";}
              else { print '#repeat resume for '.$outputfile."-end\n";}
              ##close(RESOUTPUT);
            } #end of resume reporting
          } # end of different $repeats
        } # end of different $post
      }
    }
  }
  print 'INFO @data[$d] = '.$data[$d]." - FINISHED\n";  
} 
#disconnect from database
if ($dbi) { $dbh->disconnect;}  
print 'INFO script finishes succesfully'."\n";      

######################################## SUBROUTINES
######################################################################################
# local time in format 0000-00-00 00:00:00
######################################################################################
sub time
{ 
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,
      $yday,$isdst)=localtime(time);
  my $format = sprintf ("%4d-%02d-%02d %02d:%02d:%02d",$year+1900, $mon+1,$mday,$hour,$min,$sec); 
  return($format);
}
           
######################################################################################
#loading variables from file into main
######################################################################################
sub readsettings #&readsettings($settingsfile) or $wMfile
{ 
  my ($file) = @_; 
  open(FI, $file);
  my $scaledPseudo = 0;
  my @split;
  while(<FI>) 
  {
    my $line = $_; 
    chomp $line;
    if (($line =~ m/^\$/ || $line =~ m/^\@/) && !($line =~ m/^\$pwmCS/)){ eval($line);}
    #take the pwmCS for the requested $ps from wM.txt
    #format :  $scaledPs = 0.111; #avg[(0.3_0.2_0.2_0.3)/sqrt(nbrSeq)]
    #format :  $pwmCS = 1.45342; # for pseudocount = 0.001 
    if ($bScaled && $line =~ m/^\$scaledPs/)
    {
      @split = split(/\s/,$line);
      $scaledPseudo = $split[2]; chop($scaledPseudo); #remove ;
      eval($line); #this will also set $scaledPS in main.
    }
    if ($line =~ m/^\$pwmCS/)
    {
      @split = split("# for pseudocount = ",$line);
      if ($bScaled) { if ($scaledPseudo == $split[1]) { eval($line);}} ##this will set $pwmCS in main.
      elsif ($ps == $split[1]){ eval($line);} ##this will set $pwmCS in main.
    }
  }
  close(FI);
}

######################################################################################
#count the number of matrices in a matrixfile
######################################################################################
sub countmatrices # ($D1) = &countmatrices($matrixfile);
{
  my ($fName) = @_; 
  open(FI, "<$fName");
  my $count = 0; my $motif = ''; my @split;
  while(<FI>)
  { 
    my $line = $_;
    if ($line =~ m/^#ID/) #line of motif #ID = box_1_1_TCGATAACnTGTT_[PWM=CS-based]
    { # for fc, both CS as W may be reported : count this only once ! %2009/12/02
      @split = split("PWM",$line);
      if (@split >1) #this means it is fc-output
      {
        if ( $split[0] eq $motif) {next;} #do not count the second W-motif 
        $motif = $split[0];
      }
      $count++;
    }
  } 
  close(FI); 
  return($count);
}

######################################################################################
#create annotated hash based on annotatie-file contents
## changed for 250_50_new, now containes start AND end information (as length can be variable)
## we assume that the file only has ONE motif with its TFBS's
###################################################################################### 
sub annotatedhash # my ($annotP, $hashref)= &annotatedhash($annotatie);
{
  my ($annotfile) = @_; 

  open(FI1, $annotfile);
  my %hash; 
  my $annotNr = 0; 
  my $seqid; my $start, my $end; 
  while (<FI1>)
  {
    my $line = $_;
    chomp($line);
    if ($line =~ m/^\#/){next;}
    # read the site-starts !!!! and site ends !!!! for this motif 
    #format (urv pos 202 [fwd] L 17 end 218)
    my @site = split(/\s+/,$line);
    if (@site > 7) ##
    { $seqid = $site[0]; $start = $site[2]; $end = $site[7]; ##
      if (exists $hash{$seqid}){ push(@{$hash{$seqid}},$start); push(@{$hash{$seqid}},$end);}
      else { $hash{$seqid} = ["$start"]; push(@{$hash{$seqid}},$end);} 
      if ($start > -1) {$annotNr++;} #track number of postitive annotated sites
    }
  }
  close(FI1);
  return ($annotNr, \%hash);
}

######################################################################################
#run motifcomparison with KL or BLiC, return ids of the correct matrices
###################################################################################### 
sub motifcomparison # my $KLoutfile = &motifcomparison($mtrxfile, $annotpwm, $ps, $posttype, @array); 
{
  my($matrices, $database, $pseudo, $postt, @par) = @_; #@KL = -t/-s/-x/-l, #@BLIC = -t/-s/-x/-l/-n/-b/-p
  my @code = (' -t ', ' -s ', ' -x ', ' -l ', ' -n ', ' -b ', ' -p');
  my @split;

  ## create output -o for motifcomparison, file will be stored in resultdir (just as information for user)
  my $outfile = $matrices;   $outfile =~ s/\.mtrx//;
  my $treshpr = $par[0]; $treshpr =~ s/\.//;
  my $MCkolomm = '_MC'.$pseudo.'_t'.$treshpr.'_s'.$par[1].'_x'.$par[2].'_l'.$par[3];
  if ( $par[3] == 1 ) 
  { my $parr;
    for (my $c = 4; $c < scalar(@par); $c++) 
    { $parr = $par[$c];
      if ($c == 4) { $MCkolomm .= '_n'.$parr;}
      if ($c == 5) { @split = split("/",$parr); $parr = $split[scalar(@split)-1]; $MCkolomm .= '_b'.$parr; $MCkolomm =~ s/\.b/b/;}
      if ($c == 6) { @split = split("/",$parr); $parr = $split[scalar(@split)-1]; $MCkolomm .= '_p'.$parr; $MCkolomm =~ s/\.d/d/;}
    } 
  }
  $outfile .= $MCkolomm.'.txt';


  #if (!(-e $outfile))
  #{
    ## for FC-output, only consider the CS-motif in case also W-motif is printed %2009/12/02
    ## for MR-output, only consider the first highest scoring motif %%2010/04/05 : CONSIDER ALL MATRICES FOR MR (top5)
    ## make a copy of matrix-file with sub-matrices %%2010/04/05 : not needed for MS and MR
    my $matrices2 = $matrices;
    if ($postt =~ m/clust/)
    { 
      $matrices2 = $matrices.'sub';
      open(OUTTFILE, ">$matrices2");
      print OUTTFILE "#INCLUSive Motif Model\n#\n";
      open(FI, "<$matrices");
      my $go = 0; my $motif = ''; my @split;
      while(<FI>)
      { 
        my $line = $_;
        if ($line =~ m/^#ID/) #line of motif #ID = box_1_CS[...]_W[...]_E=0.329804_nbrInst=8_[PWM=CS-based]
        { # for fc, both CS as W may be reported : copy only CS-based (printed first)
          @split = split("PWM",$line);
          if (@split > 1) #this means it is FC-output
          {
            if ( $split[0] eq $motif) {$go = 0; next;} #FC : do not store the second W-motif 
            $motif = $split[0]; 
          }
          $go = 1;
        }
        if ($go) { print OUTTFILE $line;}
      } 
      close(FI); 
      close(OUTTFILE);
    }

    ## run the MC commando
    $command = $driverpath.'applications/MotifComparison -m '.$matrices2.' -o '.$outfile.' -d '.$database;
    for(my $p = 0; $p < scalar(@par); $p++) { $command .= $code[$p].$par[$p];}
    system($command);
    #print '#command : '.$command."\n";

    ## delete the matrices2-file %2009/12/02
    if ($postt =~ m/clust/) { $command = 'rm '.$matrices2; system($command);}
  #}

  return ($outfile);
}


######################################################################################
# extract from MStxtfile those motifs that belong to MRtxtfile, in the correct ranked order
######################################################################################
sub gettfbsinfo #$txt2file = &gettfbsinfo($txtfile);
{
  my ($MRfile) = @_; 
  my @splitt= split("_MR", $MRfile); my $MSfile = $splitt[0].'.txt';
  if (!(-e $MSfile))
  { print 'WARNING '.$MSfile." : file not found - DEBUG sub gettfbsinfo - exit!\n"; exit(-1);}

  my $txt2 = $MRfile.'MRsub';

  my @split; my $motifid; my $id; my $go = 0;

  open(FOUT, ">$txt2");
  print FOUT '#INCLUSive GFF File'."\n";

  # read through MRfile
  open(FI, "<$MRfile");
  while(<FI>)
  { 
    my $line = $_;
    if ($line =~ m/^\t/ || $line =~ m/^\#/) {next;}
    if ($line =~ m/^\d+\)/) #line of ranked motif 1)box_31_1_wkwACTGTATATnnAynCAG	126.981
    { @split = split(/\)|\s+|\t+/,$line);
      $motifid = $split[1]; 
      # find this motif in MStextfile and copy all tfbs lines
      $go = 0;
      open(FI2, "<$MSfile");
      while(<FI2>)
      { 
        my $line = $_;
        if ($line =~ m/^\#id/) 
        { #(#id: box_31_1_wkwACTGTATATnnAynCAG	cons...)
          @split = split(/\t+|\s+/,$line);
          $id = $split[1];
          if ($id eq $motifid) {$go = 1;} elsif ($go == 1){ last;}
        }
        if ($go) #print instances
        { print FOUT $line;}   
      }
      close(FI2);
      if (!$go)
      { print 'WARNING '.$motifid.' : not found in '.$MSfile." - DEBUG sub gettfbsinfo - exit!\n"; exit(-1);}
    }
  } 
  close(FI);
  close(FOUT);
 
  return($txt2); 
}

######################################################################################
# analyse the motifs based on heuristic TFBS comparison : where pvv/sens > 50% or one > 80%
# changed in accordance to the double length of \%annothash with start and end tfbs information.
# also track id of the motif with best Fvalue
###################################################################################### 
sub TFBScomparison #my ($TFBSoutfile, $topTFBSid, @allTFBSids) = &TFBScomparison($txt2file,$width,$annotP,$annothashref,\@resume, $posttype,$txtfile );
{
  my ($txt, $width, $nbrrTFBS, $refhash, $refresume, $postt,$MRtxtfile ) = @_;
  my @correctRE = ();
  my $correctTop = 'NULL';
  my $outputfile = $txt.'TFBS';
  #remove earlier file in this testfaze !
  if (-e $outputfile) { $command = 'rm -f '.$outputfile; system($command);}

  my @allQ = (0,0,0); #F,pvv,sens
  my $correct = 0; my $topF = 0; my $FF; 

  my $truee; my $aisss;
  my $LLscoree = -1;
  my $CSscoree = -1;
  my $tpp; my $fpp; my $aiss11; my $aiss00; my $aoccc; my $simm; my $rankk = 0;
  my $probocc;
  #some variables
  my $motifidd = 'NULL'; my $go = 0; my $readtfbs = 0; my $countProb = 0;

  # also track top1, topCS, topsim, topaocc id's
  my $top1id = 'NULL';  my $topCSid = 'NULL';  my $topsimid = 'NULL';  my $topaoccid = 'NULL';
  my $first = 1; my $topCS = 0; my $topsim = 0; my $topaocc = 0; 

  # read through txt-file of all motifs with the TFBS information
  open(FINPUT, "<$txt");
  while(<FINPUT>)
  {
    my $line = $_; 
    chomp($line);
    if ($line =~ m/^\#/ && !($line =~ m/^\#id/)) {$readtfbs = 0; next;} #2009/12/02 to reset in case of fc-file !
    if ($line =~ m/^\#id/) 
    { 
      #first process the previously loaded motif :
      ## print computed info to $outfile
      ## add the motif to correct motifs "@correctRE" if so
      ## keep track of topTFBSid
      if ($motifidd ne 'NULL' && $go == 1)
      { $rankk++; 
        if ($countProb > 0) { $aoccc /= $countProb;}
        ($correct, $FF) = &processquality($tpp, $fpp, $nbrrTFBS, $aiss11, $aiss00, $motifidd, $CSscoree, $LLscoree, $outputfile, $simm, $aoccc, $rankk, $postt, \@allQ);
        if ($correct){ push(@correctRE, $motifidd);}
        if ($FF > $topF){ $correctTop = $motifidd; $topF = $FF; }
        if ($first){$top1id = $motifidd; $first = 0;}
        if ($CSscoree > $topCS){$topCSid = $motifidd; $topCS = $CSscoree;}
        if ($simm > $topsim){$topsimid = $motifidd; $topsim = $simm;}
        if ($aoccc > $topaocc) { $topaoccid = $motifidd; $topaocc = $aoccc;}
      }

      # reset for the new motif
      $tpp = 0; $fpp = 0; $go = 1; $readtfbs = 1;  $aiss11 = 0; $aiss00 = 0;
      $CSscoree = 0; $LLscoree = 0; $motifidd = 'NULL';
      $simm = 0; $aoccc = 0; $countProb = 0;
      #extract id of the motif 
      $motifidd = &extractid($line); 
      if ($motifidd eq 'NULL') { $go = 0; $readtfbs = 0;} 
      #extract/track score of the motif
      if ($go)
      { $CSscoree = &extractscore($line); #CS for MS/MR/FC 
        # for motifranking/motifsampler extract LL 
        if (!($postt =~ m/clust/)){$LLscoree = &extractLLscore($line); } 
        # for MotifRanking, extract number of similars
        if ($postt =~ m/rank/){$simm = &simils($MRtxtfile, $motifidd);}
      }
      next;
    }
    # process the instances of this motif
    if ($readtfbs) 
    { ($truee, $aisss, $probocc) = &validateTFBS($line, $width, $refhash);
      if ($truee == -1) { $go = 0; $readtfbs =0; next;}
      $aoccc += $probocc ;
      $countProb += $aisss; 
      if ($truee == 1) { $tpp++; $aiss11 += $aisss;next;}
      if ($truee == 0) { $fpp++; $aiss00 += $aisss;}
    }
  }
  close(FINPUT);

  #also process the very last loaded motief 
  if ($motifidd ne 'NULL' && $go == 1)
  { $rankk++;
    if ($countProb > 0) { $aoccc /= $countProb;}
    ($correct, $FF) = &processquality($tpp, $fpp, $nbrrTFBS, $aiss11, $aiss00, $motifidd, $CSscoree, $LLscoree, $outputfile, $simm, $aoccc, $rankk, $postt, \@allQ);
    if ($correct){ push(@correctRE, $motifidd);}
    if ($FF > $topF){ $correctTop = $motifidd; $topF = $FF; }
    if ($first){$top1id = $motifidd; $first = 0;}
    if ($CSscoree > $topCS){$topCSid = $motifidd; $topCS = $CSscoree;}
    if ($simm > $topsim){$topsimid = $motifidd; $topsim = $simm;}
    if ($aoccc > $topaocc) { $topaoccid = $motifidd; $topaocc = $aoccc;}
  }

  #store allTFBs D11/F/pvv/sens in resume and print to DBI
  my $d11 = scalar(@correctRE);
  $$refresume[2] += $d11; # D11
  # scale and store the allQ in resume
  if ($postt =~ m/none/)
  { for (my $a = 0; $a < 3; $a++)
    { if ($d11 > 0) {$allQ[$a] /=  $d11; }
      $$refresume[$a+3] += $allQ[$a];
    }
    my $af = $allQ[0]; my $ap = $allQ[1];my $as = $allQ[2];
    $command = "update $table set MSaT_D11 = $d11  ,  MSat_F = $af   ,  MSaT_pvv = $ap   ,   MSaT_sens = $as where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
  }
  else #rank/clust
  { if ($postt =~ m/rank/)
    { $command = "update $table set MRaT_D11 = $d11 where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";}
    else
    { $command = "update $table set FCaT_D11 = $d11 where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";}
  }

  $dbh->do("$command");

  # add the top information to resume and print top information to performance table
  if ($correctTop ne 'NULL') {&storeTOPinfo($correctTop, $outputfile, $postt, 'topTFBS', $refresume);}
  if (!($postt =~ m/none/) && $top1id ne 'NULL') { &storeTOPinfo($top1id, $outputfile, $postt, 'top1', $refresume);}
  if (!($postt =~ m/none/) && $topCSid ne 'NULL') { &storeTOPinfo($topCSid, $outputfile, $postt, 'topCS', $refresume);}
  if ($postt =~ m/rank/ && $topsimid ne 'NULL') { &storeTOPinfo($topsimid, $outputfile, $postt, 'topsim', $refresume);}
  if ($postt =~ m/clust/&& $topaoccid ne 'NULL') { &storeTOPinfo($topaoccid, $outputfile, $postt, 'topaocc', $refresume);}
 
  return($outputfile, $correctTop , $top1id, $topCSid, $topsimid, $topaoccid, @correctRE);
}

######################################################################################
# process the quality of the motif : evaluate if correct and store information
###################################################################################### 
sub processquality #($bool, $fvalue) = &processquality($tpp, $fpp, $nbrrTFBS, $aiss11, $aiss00, $motifidd, $CSscoree, $LLscoree, $outputfile, $simm, $aoccc, $rankk, $postt, \@allQ);
{
  my ($tp, $fp, $nbrTFBS, $aiss1, $aiss0, $motifid, $CSscore, $LLscore, $outfile, $Sim, $Aocc, $Rank, $po, $refallQ) = @_;

  open(FOTFBS, ">>$outfile"); 

  ## always compute the quality of the motif (regardless if correct)
  my $Pvv = 0; my $fvalue = 0; my $Sens = $tp/$nbrTFBS;
  if ($tp+$fp > 0) { $Pvv = $tp/($tp+$fp);} 
  if (($Pvv+$Sens) > 0){$fvalue = 2*$Pvv*$Sens/($Pvv+$Sens);}
  if ($tp > 0) {$aiss1 /= $tp;}
  if ($fp > 0) {$aiss0 /= $fp;}

  #analyse if it is a correct value for TFBS-heuristic
  my $bool = 1;
  if ( &heuristicaccept($tp, $fp, $nbrTFBS))
  { # add the quality to temporary resume for bNone
    if ($po =~ m/none/) { $$refallQ[0] += $fvalue; $$refallQ[1] += $Pvv; $$refallQ[2] += $Sens;}
  }
  else
  { $bool = 0; print FOTFBS '##';} ## to show that it is in fact an incorrect motief
    

  # print the quality for the all the motifs
  print FOTFBS 'id='.$motifid.' F='.$fvalue.' pvv='.$Pvv.' sens='.$Sens.' CS='.$CSscore;
  if (!($po =~ m/clust/)){ print FOTFBS ' LL='.$LLscore;}
  if ($po =~ m/rank/){ print FOTFBS ' sim='.$Sim;}
  if ($po =~ m/clust/){ print FOTFBS ' aocc='.$Aocc;}
  if (!($po =~ m/none/)){ print FOTFBS ' rank='.$Rank;}
  if ($po =~ m/clust/){ print FOTFBS ' aISS1='.$aiss1.' aISS0='.$aiss0;}
  print FOTFBS "\n";

  close(FOTFBS);
  return($bool, $fvalue);
}

######################################################################################
# 
###################################################################################### 
sub extractid #$motifid = &extractid($line);
{
  my $linee = $_; 
  my $id = 'NULL';
  #(#id: box_1_CS[13,TCGATAACnTGTT,1.71433]_E=0.456723_nbrInst=10)
  #(#id: box_1_1_TCGATAACnTGTT consensus: TCGATAACnTGTT sequences: 10 instances: 10 cs: 1.55 ic: 1.55 ll: 152.32)
  my @split = split(/\t+|\s+/,$linee); 
  if (@split > 1){ $id = $split[1];} 
  return($id);
}
######################################################################################
# 
###################################################################################### 
sub extractscore #$score = &extractscore($line);
{
  my $linee = $_; 
  chomp($linee);
  my $scoree = -1;
  my @split;
  #(#id: box_1_CS[13,TCGATAACnTGTT,1.71433]_E=0.456723_nbrInst=10)
  #(#id: box_1_1_TCGATAACnTGTT consensus: TCGATAACnTGTT sequences: 10 instances: 10 cs: 1.55 ic: 1.55 ll: 152.32)
  if ($bClust) # case of txt-fc file
  { @split = split(/\,|\]/,$linee);
    if (scalar(@split) > 2)
    { $scoree = $split[2];} # CS 
  }
  else # case of txt-sampling file
  { @split = split(/\t+|\s+/,$linee);
    if (@split > 9)
    { $scoree = $split[9];} #cs
  }
  return($scoree);
}
######################################################################################
# 
###################################################################################### 
sub extractLLscore #$score = &extractLLscore($line);
{
  my $linee = $_; 
  chomp($linee);
  my $scoree = -1;
  my @split;
  #(#id: box_1_1_TCGATAACnTGTT consensus: TCGATAACnTGTT sequences: 10 instances: 10 cs: 1.55 ic: 1.55 ll: 152.32)
  #if ($bClust) # not applicable
  @split = split(/\t+|\s+/,$linee);
  if (@split > 13)
  { $scoree = $split[13];} #ll
  return($scoree);
}
######################################################################################
# 
###################################################################################### 
sub heuristicaccept  #bool = &heuristicaccept($tp, $fp, $nbrTFBS); 
{
  my ($TP, $FP, $tot) = @_;
  my $Pvv = 0; 
  if ($TP+$FP > 0) { $Pvv = $TP/($TP+$FP)}; 
  my $Sens = $TP/$tot;
  if ( (($Pvv>0.5)&&($Sens>0.5)) || (($Pvv > 0.8)||($Sens>0.8)) ) {return(1);}
  return(0);
}
######################################################################################
# 
###################################################################################### 
sub validateTFBS #($truee, $aisss, $probocc) = &validateTFBS($line, $width, $refhash);
{ 
      my ($linee, $w, $reffhash) = @_;
#print 'sub validate TFBS line = '.$linee."\n";
      my $true = 0; my $aiss = 0;
      my $Tstart; my $Tend;
      my $proboc = 0;

      # read the seq-id and site-start ## and site-END
      my @site = split(/\t+|\s+/,$linee);
      my @split;
      my $go = 1;
      my $seq = -1; my $start = -1; my $end = -1; 
      if ($bClust) # (SEQ0_420_432_+_TCGATAACCTGTT_(shift=0,occ=59)	1)
      { if ( @site > 1)
        { 
          @split = split(/_/, $site[0]);
          if ( @split > 5)
          { $seq = $split[0]; $start = $split[1]; $end = $split[2]; $aiss = $site[1];
            my @splitt = split(/occ=/, $split[5]); $proboc = $splitt[1]; chop($proboc); #remove ')'
            $proboc *= $aiss; # membership-probability * number of occurences
          } else {$go = 0;} #2009/12/02 reset in case of wrong format
        } else {$go = 0;} #2009/12/02 reset in case of wrong format
      }
      else #(SEQ0 MotifSampler misc_feature 420 432 3.02613e+06 + . id "box_1_1_TCGATAACnTGTT"; site "TCGATAACCTGTT";)
      { if ( @site > 5)
        { $seq = $site[0];$start = $site[3];$end = $site[4];} else {$go = 0;} #2009/12/02 reset in case of wrong format
      }
      if (!$go) {$true = -1;} #2009/12/02 reset (stop for this motif) and read on to next motif
      else
      {
        ## now compare with ($refhash) from annotatie-file 
        $true = 0;
        if (exists $$reffhash{$seq})
        { 
          for my $key ( keys %$reffhash ) 
          { if ($seq eq $key) 
            { for(my $s=0; $s < scalar(@{ $$reffhash{$seq} }); $s++)
              { $Tstart = $$reffhash{$seq}[$s]; $s++;
                $Tend = $$reffhash{$seq}[$s]; 
                if ($Tstart < 0) { last;} #this sequence has no annotated sites
                if ( ($start > ($Tstart - int($w/2))) && ($start < ($Tend - int($w/2) )))
                {$true = 1; last;}
              }
            }
          }
        } 
      }
      return ($true, $aiss, $proboc);
}

######################################################################################
# get the number of similars of inputmotif for MR

###################################################################################### 
sub simils # $sim = &simils($txtfile, $id);
{
  my ($txt, $id) = @_;

  #some variables
  my $motifid = 'NULL'; my $go = 0; my $simils; 
  my @split; 
 

  ## read through MRtxtfile  
  open(FI, "<$txt");
  while(<FI>)
  { 
    my $line = $_;
    if ($line =~ m/^\t/ || $line =~ m/^\#/) {next;}
    if ($line =~ m/^\d+\)/) #line of ranked motif 1)box_31_1_wkwACTGTATATnnAynCAG	126.981
    { @split = split(/\)|\s+|\t+/,$line);
      $motifid = $split[1];
      if ($motifid eq $id) {$go = 1;}
      next;
    }
    if ($go && $line =~ m/^\=\=\>\sTotal/) # similars information ==> Total: 22
    {
      @split = split(/\s+|\t+/,$line);
      $simils = $split[2];
      last;
    }
  }  
  close(FI); 
  return($simils);
}
######################################################################################
# get and store the MC-distance-scores for the given id's
###################################################################################### 
sub getMCscore #($kl) = &getMCscore($topTFBSid, $KLoutfile); idem voor BLiC
{
  my ($id, $file) = @_;
  $distance = 0; 
  my @score;
  my $motifid = 'NULL';
#KLformat : #query:query_consensus	match:match_consensus	score;(shiftm-d,shiftd-m)
#box_1_1_ATTCCTACnnnTGTArGA:ATTCCTACnnnTGTArGA	box_ann1_ps0.001_TTCCTACAsyTGTAAGAAw:TTCCTACAsyTGTAAGAAw	0.270935;(-1,1)
#BLiCformat : query:query_consensus	match:match_consensus	score;shift;overlap;strand	p-value;common;(n:avg,stdev)
#box_1_1_ATTCCTACnnnTGTArGA:ATTCCTACnnnTGTArGA	box_ann1_ps0.001_TTCCTACAsyTGTAAGAAw:TTCCTACAsyTGTAAGAAw	20.5;-1;17;fwd	5.3301e-07;0;(n=18:9.07,1.06)
  open(INPUT2, "<$file");
  while (<INPUT2>) 
  {
    my $line = $_;
    #print '$line = '.$line;
    if ($line =~ m/box/) #read all lines (with or without ##)
    {
      my @result = split(/\t/, $line);
      my @idd = split(/\:/, $result[0]);
      $motifid = $idd[0]; 
      $motifid =~ s/\#\#//;
      if ( $motifid ne $id ) { next; }
      # read the score of motif (p-value, KLdist)
      if ( scalar(@result) == 3 ) #KLdist
      { @score = split(/\;/,$result[2]);} 
      else #p-value
      { @score = split(/\;/,$result[3]);} 
      $distance = $score[0];
    }
  }
  close(INPUT2);
  return($distance);
}

######################################################################################
# add the top information to resume and print top information to performance table
###################################################################################### 
sub storeTOPinfo # &storeTOPinfo($correctTop, $outputfile, $postt, 'topTFBS', $refresume);
{
  my ($topid, $file, $po, $toptype, $refresume) = @_;

#print 'topid = '.$topid.' - toptype = '.$toptype.' - posttype = '.$po."\n";

  my @split; my $line; my $found = 0;
  my $id;
  open(FII, "<$file");
  while(<FII>)
  {
    $line = $_; 
    chomp($line);
    if ($line =~ m/id/) 
    { @split = split(/\s+/,$line);
      $id = $split[0]; $id =~ s/\#\#//; $id =~ s/id=//;
#print 'id = '.$id."\n";
      if ($id eq $topid) {$found = 1; last;}
    }
  }
  if (!$found)
  { print 'Error- sub storeTOPinfo, something wrong in script id = '.$id.' - topid = '.$topid.'- debug - exit'."\n"; exit(-1);}

  my $FF = $split[1]; $FF =~ s/F=//; 
  my $p = $split[2]; $p =~ s/pvv=//;
  my $s = $split[3]; $s =~ s/sens=//;
  my $C = $split[4]; $C =~ s/CS=//;
  if ($po =~ m/none/)
  { 
    my $L = $split[5]; $L =~ s/LL=//;
    $command = "update $table set MStT_F = $FF , MStT_pvv = $p  ,  MStT_sens = $s  ,  MStT_CS = $C ,  MStT_LL = $L ,  MStT_id = '$topid' where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
    $$refresume[20] += $FF; $$refresume[21] += $p; $$refresume[22] += $s; $$refresume[23] += $C; $$refresume[24] += $L;
  }
  elsif ($po =~ m/rank/)
  { 
    my $L = $split[5]; $L =~ s/LL=//;
    my $S = $split[6]; $S =~ s/sim=//;
    my $r = $split[7]; $r =~ s/rank=//;
    if ($toptype eq 'topTFBS') 
    { $command = "update $table set MRtT_F = $FF  ,  MRtT_pvv = $p  ,  MRtT_sens = $s  ,  MRtT_CS = $C  ,  MRtT_LL = $L  ,  MRtT_id = '$topid'  ,  MRtT_sim = $S  ,  MRtT_rank = $r where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
    $$refresume[3] += $FF; $$refresume[4] += $p; $$refresume[5] += $s; $$refresume[8] += $C; $$refresume[7] += $L;$$refresume[6] += $S;$$refresume[9] += $r;
    }
    elsif ($toptype eq 'top1') 
    { $command = "update $table set MRtLL_F = $FF  ,  MRtLL_pvv = $p  ,  MRtLL_sens = $s  ,  MRtLL_CS = $C  ,  MRtLL_LL = $L  ,  MRtLL_id = '$topid'  ,  MRtLL_sim = $S  ,  MRtLL_rank = $r where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
    $$refresume[12] += $FF; $$refresume[13] += $p; $$refresume[14] += $s; $$refresume[17] += $C; $$refresume[16] += $L;$$refresume[15] += $S;$$refresume[18] += $r;
    }
    elsif ($toptype eq 'topCS') 
    { $command = "update $table set MRtCS_F = $FF  , MRtCS_pvv = $p ,  MRtCS_sens = $s  ,  MRtCS_CS = $C  ,  MRtCS_LL = $L  ,  MRtCS_id = '$topid'  ,  MRtCS_sim = $S ,  MRtCS_rank = $r where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
    $$refresume[21] += $FF; $$refresume[22] += $p; $$refresume[23] += $s; $$refresume[26] += $C; $$refresume[25] += $L;$$refresume[24] += $S;$$refresume[27] += $r;
    }
    elsif ($toptype eq 'topsim') 
    { $command = "update $table set MRtsim_F = $FF ,  MRtsim_pvv = $p ,  MRtsim_sens = $s ,  MRtsim_CS = $C  , MRtsim_LL = $L  ,  MRtsim_id = '$topid'  ,  MRtsim_sim = $S  ,  MRtsim_rank = $r where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
    $$refresume[30] += $FF; $$refresume[31] += $p; $$refresume[32] += $s; $$refresume[35] += $C; $$refresume[34] += $L;$$refresume[33] += $S;$$refresume[36] += $r;
    }
  }
  else #bClust
  { 
    my $a = $split[5]; $a =~ s/aocc=//;
    my $r = $split[6]; $r =~ s/rank=//;
    my $a1 = $split[7]; $a1 =~ s/aISS1=//;
    my $a0 = $split[8]; $a0 =~ s/aISS0=//;
    if ($toptype eq 'topTFBS') 
    {$command = "update $table set FCtT_F = $FF  ,  FCtT_pvv = $p  ,  FCtT_sens = $s  ,  FCtT_CS = $C  ,  FCtT_aocc = $a  ,  FCtT_id = '$topid'  ,  FCtT_rank = $r  ,  FCtT_aISS1 = $a1  ,  FCtT_aISS0 = $a0 where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
    $$refresume[3] += $FF; $$refresume[4] += $p; $$refresume[5] += $s; $$refresume[8] += $a0; $$refresume[7] += $a1;$$refresume[6] += $a;$$refresume[10] += $r;$$refresume[9] += $C;
    }
    elsif ($toptype eq 'top1')
    {$command = "update $table set FCt1_F = $FF ,  FCt1_pvv = $p  , FCt1_sens = $s  ,  FCt1_CS = $C  ,  FCt1_aocc = $a  , FCt1_id = '$topid'  ,  FCt1_rank = $r  ,  FCt1_aISS1 = $a1  ,  FCt1_aISS0 = $a0 where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
    $$refresume[13] += $FF; $$refresume[14] += $p; $$refresume[15] += $s; $$refresume[18] += $a0; $$refresume[17] += $a1;$$refresume[16] += $a;$$refresume[20] += $r;$$refresume[19] += $C;
    }
    elsif ($toptype eq 'topCS')
    {$command = "update $table set FCtCS_F = $FF  ,  FCtCS_pvv = $p  ,  FCtCS_sens = $s  , FCtCS_CS = $C ,  FCtCS_aocc = $a  ,  FCtCS_id = '$topid' ,  FCtCS_rank = $r ,  FCtCS_aISS1 = $a1  ,  FCtCS_aISS0 = $a0 where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
    $$refresume[23] += $FF; $$refresume[24] += $p; $$refresume[25] += $s; $$refresume[28] += $a0; $$refresume[27] += $a1;$$refresume[26] += $a;$$refresume[30] += $r;$$refresume[29] += $C;
    }
    elsif ($toptype eq 'topaocc')
    {$command = "update $table set FCtaocc_F = $FF ,  FCtaocc_pvv = $p  ,  FCtaocc_sens = $s ,  FCtaocc_CS = $C  ,  FCtaocc_aocc = $a  ,  FCtaocc_id = '$topid'  ,  FCtaocc_rank = $r ,  FCtaocc_aISS1 = $a1  ,  FCtaocc_aISS0 = $a0 where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
    $$refresume[33] += $FF; $$refresume[34] += $p; $$refresume[35] += $s; $$refresume[38] += $a0; $$refresume[37] += $a1;$$refresume[36] += $a;$$refresume[40] += $r;$$refresume[39] += $C;
    }
  }
  $dbh->do("$command");
  return;
}
###############################################################################################
  #### evaluate the results of MotifComparison (resp KL and BLiC) : topKL/BLiC and allKL/BLiC
################################################################################################
sub MCevaluation # &MCevaluation($KLoutfile, $BLiCoutfile, $TFBSoutfile, \@resume);
{
  my($file1, $file2, $TFBSfile, $refresume) = @_;

  #### evaluate the results of MotifComparison (resp KL and BLiC) :
  ## read through MC-file : count not-## lines (D11), sum both the KL and BliC, and get/sum the F/pvv/sens
  ## track the id with the lowest distance (may be incorrect!) and store/print F/pvv/sens/CS/LL/KL/BLiC/id
  my $d11 = 0; my $aF = 0; my $apvv = 0; my $asens = 0; my $aKL = 0; my $aBLiC = 0;
  my $topid = 'NULL'; my $topDistance = 100; my $topF = 0; my $toppvv = 0; my $topsens = 0; my $topCS = 0; my $topLL = 0; my $otherDist = 0;
  my $FF; my $p; my$s; my$C; my$L;
  my $bKL = 0;

#KLformat : #query:query_consensus	match:match_consensus	score;(shiftm-d,shiftd-m)
#box_1_1_ATTCCTACnnnTGTArGA:ATTCCTACnnnTGTArGA	box_ann1_ps0.001_TTCCTACAsyTGTAAGAAw:TTCCTACAsyTGTAAGAAw	0.270935;(-1,1)
#BLiCformat : query:query_consensus	match:match_consensus	score;shift;overlap;strand	p-value;common;(n:avg,stdev)
#box_1_1_ATTCCTACnnnTGTArGA:ATTCCTACnnnTGTArGA	box_ann1_ps0.001_TTCCTACAsyTGTAAGAAw:TTCCTACAsyTGTAAGAAw	20.5;-1;17;fwd	5.3301e-07;0;(n=18:9.07,1.06)
  my @score; my $score1; my $score2; my $motifid; my $tracked = 0; my @split;

#print "file1 = $file1 - file2 = $file2\n";

  my $bCorrect = 0; 
  open(INPUT1, "<$file1"); 
  open(INPUT2, "<$file2"); 
  my $line1; my $line2; my @result; my@id; my @result2; my @id2; my $idd2;
  while ( <INPUT1>) 
  {
    $line1 = $_;
#print "line 1 = $line1";
    $line2 = <INPUT2>; #read the same line in other file
#print "line 2 = $line2";
    ##if ($line1 =~ m/^\#/) {next;} # masked lines for distant matrices / also read incorrect matrices to track topKL/topBLiC.
    if ($line1 =~ m/box/) # this is a matrix
    {
      $bCorrect = 0; # reset
      @result = split(/\t/, $line1);
      @id = split(/\:/, $result[0]);
      $motifid = $id[0]; 
      if ($motifid =~ m/^\#/) {$motifid =~ s/\#\#//; } else {$bCorrect = 1; $d11++; }#remove ## prefix
      # read the score of this motif (p-value, KLdist) - may be correct or incorrect !
      if ( scalar(@result) == 3 ) #KLdist
      { $bKL = 1; 
        @score = split(/\;/,$result[2]); $score1 = $score[0]; if ($bCorrect) {$aKL += $score1;}
        # get the BLiC score for this id 
        @result2 = split(/\t/, $line2);
        @id2 = split(/\:/, $result2[0]); $idd2 = $id2[0]; $idd2 =~ s/\#\#//;
        if ($motifid ne $idd2) {print 'ERROR - sub MCevalutation : error in file-reading1 - debug $motifid = '.$motifid.' - idd2 = '.$idd2.' - exit'."\n"; exit(-1);}
        @score = split(/\;/,$result2[3]); $score2 = $score[0]; if ($bCorrect) {$aBLiC += $score2;} 
      } 
      else 
      { $bKL = 0; 
        @score = split(/\;/,$result[3]); $score1 = $score[0]; if ($bCorrect) {$aBLiC += $score1;}
        # get the KL score for this id
        @result2 = split(/\t/, $line2);
        @id2 = split(/\:/, $result2[0]); $idd2 = $id2[0]; $idd2 =~ s/\#\#//;
        if ($motifid ne $idd2) {print 'ERROR - sub MCevalutation : error in file-reading1 - debug $motifid = '.$motifid.' - idd2 = '.$idd2.' - exit'."\n"; exit(-1);}
        @score = split(/\;/,$result2[2]); $score2 = $score[0]; if ($bCorrect) {$aKL += $score2; }
      }
      # track if it is the smallest score
      if ($score1 < $topDistance)
      { $topid = $motifid; $topDistance = $score1; $otherDist = $score2; $tracked = 1;}
      else { $tracked = 0;}

      # also get the F/pvv/sens for this id
#print "find $motifid in file $TFBSfile\n";
      my $found = 0; my $idd = 'NULL'; 
      open(FII, "<$TFBSfile");
      while(<FII>)
      {
        my $line3 = $_; 
        chomp($line3);
        if ($line3 =~ m/id/) 
        { @split = split(/\s+/,$line3);
          $idd = $split[0]; $idd =~ s/id=//; $idd =~ s/\#\#//;
#print "id = $idd\n";
          if ($idd eq $motifid ) {$found = 1; last;}
        }
      }
      if (!$found)
      { print 'Error- sub MCevalutation, something wrong in script - debug - exit'."\n"; exit(-1);}
      # get the information
      $FF = $split[1]; $FF =~ s/F=//; $p = $split[2]; $p =~ s/pvv=//;$s = $split[3]; $s =~ s/sens=//;$C = $split[4]; $C =~ s/CS=//;$L = $split[5]; $L =~ s/LL=//;
      # store the all information
      if ($bCorrect) {$aF += $FF; $apvv += $p; $asens += $s;}
      # and the top information
      if ($tracked)
      { $topF = $FF; $toppvv = $p; $topsens = $s; $topCS = $C; $topLL = $L;}   
      close(FI); 
    }
  }
  close(INPUT1);
  close(INPUT2);

  if ($d11 > 0) # else all $aF etc will be zero)
  { $aF /= $d11;  $apvv /= $d11; $asens /= $d11; $aKL /= $d11; $aBLiC /= $d11;}

  ## add to resume and print to dbi
  if ($bKL)
  { #topKL
    if ($topid ne 'NULL')
    {
      $command = "update $table set MStKL_F = $topF  ,  MStKL_pvv = $toppvv  ,  MStKL_sens = $topsens  ,  MStKL_CS = $topCS  ,  MStKL_LL = $topLL  ,  MStKL_id = '$topid'  , MStKL_KL = $topDistance  ,  MStKL_BLiC = $otherDist where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
      $$refresume[27] += $topF; $$refresume[28] += $toppvv; $$refresume[29] += $topsens; $$refresume[30] += $topCS; $$refresume[31] += $topLL;
      $$refresume[32] += $topDistance; $$refresume[33] += $otherDist;
      $dbh->do("$command");
    }
    #allKL
    if ($d11 > 0)
    {
      $command = "update $table set MSaKL_D11 = $d11  ,  MSaKL_F = $aF ,  MSaKL_pvv = $apvv  ,  MSaKL_sens = $asens  ,  MSaKL_KL = $aKL ,  MSaKL_BLiC = $aBLiC where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
      $$refresume[8] += $d11; $$refresume[9] += $aF; $$refresume[10] += $apvv; $$refresume[11] += $asens; $$refresume[12] += $aKL;
      $$refresume[13] += $aBLiC;
      $dbh->do("$command"); 
    }
  }
  else
  { #topBLiC
    if ($topid ne 'NULL')
    {
      $command = "update $table set MStBLiC_F = $topF  ,  MStBLiC_pvv = $toppvv  , MStBLiC_sens = $topsens  ,  MStBLiC_CS = $topCS  ,  MStBLiC_LL = $topLL  ,  MStBLiC_id = '$topid'  , MStBLiC_KL = $otherDist  ,  MStBLiC_BLiC = $topDistance where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
      $$refresume[34] += $topF; $$refresume[35] += $toppvv; $$refresume[36] += $topsens; $$refresume[37] += $topCS; $$refresume[38] += $topLL;
      $$refresume[39] += $otherDist; $$refresume[40] += $topDistance;
      $dbh->do("$command");
    }
    #allBLiC
    if ($d11 > 0)
    {
      $command = "update $table set MSaBLiC_D11 = $d11  ,  MSaBLiC_F = $aF  ,  MSaBLiC_pvv = $apvv  ,  MSaBLiC_sens = $asens  ,  MSaBLiC_KL = $aKL  ,  MSaBLiC_BLiC = $aBLiC where DateTime = '$now' and TF = '$datai' and bgm = '$bgm' and prior = '$prior' and search = '$method' and MSpar = '$MSkolom' and MRpar = '$MRkolom' and FCpar = '$FCkolom' and MCKLpar = '$MCKLkolom' and MCBLiCpar = '$MCBLiCkolom'";
      $$refresume[14] += $d11; $$refresume[15] += $aF; $$refresume[16] += $apvv; $$refresume[17] += $asens; $$refresume[18] += $aKL;
      $$refresume[19] += $aBLiC; 
      $dbh->do("$command");
    }
  }
  return;
}

